@@ -24,12 +24,19 @@
  
 #include "linphonecore.h"
 #include "private.h"
-#include <eXosip2/eXosip.h>
-#include <osipparser2/osip_message.h>
 #include "lpconfig.h"
 
-extern LinphoneProxyConfig *linphone_core_get_proxy_config_from_rid(LinphoneCore *lc, int rid);
+/**
+ * @addtogroup authentication
+ * @{
+**/
 
+/**
+ * Create a LinphoneAuthInfo object with supplied information.
+ *
+ * The object can be created empty, that is with all arguments set to NULL.
+ * Username, userid, password and realm can be set later using specific methods.
+**/
 LinphoneAuthInfo *linphone_auth_info_new(const char *username, const char *userid,
 				   										const char *passwd, const char *ha1,const char *realm)
 {
@@ -40,10 +47,42 @@ LinphoneAuthInfo *linphone_auth_info_new(const char *username, const char *useri
 	if (ha1!=NULL && (strlen(ha1)>0)) obj->ha1=ms_strdup(ha1);
 	if (realm!=NULL && (strlen(realm)>0)) obj->realm=ms_strdup(realm);
 	obj->works=FALSE;
-	obj->first_time=TRUE;
 	return obj;
 }
 
+static LinphoneAuthInfo *linphone_auth_info_clone(const LinphoneAuthInfo *ai){
+	LinphoneAuthInfo *obj=ms_new0(LinphoneAuthInfo,1);
+	if (ai->username) obj->username=ms_strdup(ai->username);
+	if (ai->userid) obj->userid=ms_strdup(ai->userid);
+	if (ai->passwd) obj->passwd=ms_strdup(ai->passwd);
+	if (ai->ha1)	obj->ha1=ms_strdup(ai->ha1);
+	if (ai->realm)	obj->realm=ms_strdup(ai->realm);
+	obj->works=FALSE;
+	obj->usecount=0;
+	return obj;
+}
+
+/**
+ * Returns username.
+**/
+const char *linphone_auth_info_get_username(const LinphoneAuthInfo *i){
+	return i->username;
+}
+
+/**
+ * Returns password.
+**/
+const char *linphone_auth_info_get_passwd(const LinphoneAuthInfo *i){
+	return i->passwd;
+}
+
+const char *linphone_auth_info_get_userid(const LinphoneAuthInfo *i){
+	return i->userid;
+}
+
+/**
+ * Sets the password.
+**/
 void linphone_auth_info_set_passwd(LinphoneAuthInfo *info, const char *passwd){
 	if (info->passwd!=NULL) {
 		ms_free(info->passwd);
@@ -52,6 +91,9 @@ void linphone_auth_info_set_passwd(LinphoneAuthInfo *info, const char *passwd){
 	if (passwd!=NULL && (strlen(passwd)>0)) info->passwd=ms_strdup(passwd);
 }
 
+/**
+ * Sets the username.
+**/
 void linphone_auth_info_set_username(LinphoneAuthInfo *info, const char *username){
 	if (info->username){
 		ms_free(info->username);
@@ -60,6 +102,9 @@ void linphone_auth_info_set_username(LinphoneAuthInfo *info, const char *usernam
 	if (username && strlen(username)>0) info->username=ms_strdup(username);
 }
 
+/**
+ * Sets userid.
+**/
 void linphone_auth_info_set_userid(LinphoneAuthInfo *info, const char *userid){
 	if (info->userid){
 		ms_free(info->userid);
@@ -68,6 +113,9 @@ void linphone_auth_info_set_userid(LinphoneAuthInfo *info, const char *userid){
 	if (userid && strlen(userid)>0) info->userid=ms_strdup(userid);
 }
 
+/**
+ * Destroys a LinphoneAuthInfo object.
+**/
 void linphone_auth_info_destroy(LinphoneAuthInfo *obj){
 	if (obj->username!=NULL) ms_free(obj->username);
 	if (obj->userid!=NULL) ms_free(obj->userid);
@@ -154,8 +202,10 @@ static int realm_match(const char *realm1, const char *realm2){
 	return FALSE;
 }
 
-
-LinphoneAuthInfo *linphone_core_find_auth_info(LinphoneCore *lc, const char *realm, const char *username)
+/**
+ * Retrieves a LinphoneAuthInfo previously entered into the LinphoneCore.
+**/
+const LinphoneAuthInfo *linphone_core_find_auth_info(LinphoneCore *lc, const char *realm, const char *username)
 {
 	MSList *elem;
 	LinphoneAuthInfo *ret=NULL,*candidate=NULL;
@@ -187,66 +237,82 @@ LinphoneAuthInfo *linphone_core_find_auth_info(LinphoneCore *lc, const char *rea
 	return ret;
 }
 
-void linphone_core_add_auth_info(LinphoneCore *lc, LinphoneAuthInfo *info)
+/**
+ * Adds authentication information to the LinphoneCore.
+ * 
+ * This information will be used during all SIP transacations that require authentication.
+**/
+void linphone_core_add_auth_info(LinphoneCore *lc, const LinphoneAuthInfo *info)
 {
-	MSList *elem;
-	char *userid;
 	LinphoneAuthInfo *ai;
-	if (info->userid==NULL || info->userid[0]=='\0') userid=info->username;
-	else userid=info->userid;
-	eXosip_lock();
-	eXosip_add_authentication_info(info->username,userid,
-				info->passwd,info->ha1,info->realm);
-	eXosip_unlock();
-	/* if the user was prompted, re-allow automatic_action */
-	if (lc->automatic_action>0) lc->automatic_action--;
+	MSList *elem;
+	
 	/* find if we are attempting to modify an existing auth info */
-	ai=linphone_core_find_auth_info(lc,info->realm,info->username);
+	ai=(LinphoneAuthInfo*)linphone_core_find_auth_info(lc,info->realm,info->username);
 	if (ai!=NULL){
-		elem=ms_list_find(lc->auth_info,ai);
-		if (elem==NULL){
-			ms_error("AuthInfo list corruption ?");
-			return;
+		lc->auth_info=ms_list_remove(lc->auth_info,ai);
+		linphone_auth_info_destroy(ai);
+	}
+	lc->auth_info=ms_list_append(lc->auth_info,linphone_auth_info_clone(info));
+	/* retry pending authentication operations */
+	for(elem=sal_get_pending_auths(lc->sal);elem!=NULL;elem=elem->next){
+		const char *username,*realm;
+		SalOp *op=(SalOp*)elem->data;
+		LinphoneAuthInfo *ai;
+		sal_op_get_auth_requested(op,&realm,&username);
+		ai=(LinphoneAuthInfo*)linphone_core_find_auth_info(lc,realm,username);
+		if (ai){
+			SalAuthInfo sai;
+			sai.username=ai->username;
+			sai.userid=ai->userid;
+			sai.realm=ai->realm;
+			sai.password=ai->passwd;
+			sal_op_authenticate(op,&sai);
+			ai->usecount++;
 		}
-		linphone_auth_info_destroy((LinphoneAuthInfo*)elem->data);
-		elem->data=(void *)info;
-	}else {
-		lc->auth_info=ms_list_append(lc->auth_info,(void *)info);
 	}
 }
 
+
+/**
+ * This method is used to abort a user authentication request initiated by LinphoneCore
+ * from the auth_info_requested callback of LinphoneCoreVTable.
+**/
 void linphone_core_abort_authentication(LinphoneCore *lc,  LinphoneAuthInfo *info){
-	if (lc->automatic_action>0) lc->automatic_action--;
 }
 
-void linphone_core_remove_auth_info(LinphoneCore *lc, LinphoneAuthInfo *info){
-	int len=ms_list_size(lc->auth_info);
-	int newlen;
+/**
+ * Removes an authentication information object.
+**/
+void linphone_core_remove_auth_info(LinphoneCore *lc, const LinphoneAuthInfo *info){
 	int i;
 	MSList *elem;
-	lc->auth_info=ms_list_remove(lc->auth_info,info);
-	newlen=ms_list_size(lc->auth_info);
-	/*printf("len=%i newlen=%i\n",len,newlen);*/
-	linphone_auth_info_destroy(info);
-	for (i=0;i<len;i++){
+	LinphoneAuthInfo *r;
+	r=(LinphoneAuthInfo*)linphone_core_find_auth_info(lc,info->realm,info->username);
+	if (r){
+		lc->auth_info=ms_list_remove(lc->auth_info,r);
+		/*printf("len=%i newlen=%i\n",len,newlen);*/
+		linphone_auth_info_destroy(r);
+		for (elem=lc->auth_info,i=0;elem!=NULL;elem=ms_list_next(elem),i++){
+			linphone_auth_info_write_config(lc->config,(LinphoneAuthInfo*)elem->data,i);
+		}
 		linphone_auth_info_write_config(lc->config,NULL,i);
 	}
-	for (elem=lc->auth_info,i=0;elem!=NULL;elem=ms_list_next(elem),i++){
-		linphone_auth_info_write_config(lc->config,(LinphoneAuthInfo*)elem->data,i);
-	}
-	
 }
 
+/**
+ * Returns an unmodifiable list of currently entered LinphoneAuthInfo.
+**/
 const MSList *linphone_core_get_auth_info_list(const LinphoneCore *lc){
 	return lc->auth_info;
 }
 
+/**
+ * Clear all authentication information.
+**/
 void linphone_core_clear_all_auth_info(LinphoneCore *lc){
 	MSList *elem;
 	int i;
-	eXosip_lock();
-	eXosip_clear_authentication_info();
-	eXosip_unlock();
 	for(i=0,elem=lc->auth_info;elem!=NULL;elem=ms_list_next(elem),i++){
 		LinphoneAuthInfo *info=(LinphoneAuthInfo*)elem->data;
 		linphone_auth_info_destroy(info);
@@ -256,78 +322,6 @@ void linphone_core_clear_all_auth_info(LinphoneCore *lc){
 	lc->auth_info=NULL;
 }
 
-void linphone_authentication_ok(LinphoneCore *lc, eXosip_event_t *ev){
-	char *prx_realm=NULL,*www_realm=NULL;
-	osip_proxy_authorization_t *prx_auth;
-	osip_authorization_t *www_auth;
-	osip_message_t *msg=ev->request;
-	char *username;
-	LinphoneAuthInfo *as=NULL;
-
-	username=osip_uri_get_username(msg->from->url);
-	osip_message_get_proxy_authorization(msg,0,&prx_auth);
-	osip_message_get_authorization(msg,0,&www_auth);
-	if (prx_auth!=NULL)
-		prx_realm=osip_proxy_authorization_get_realm(prx_auth);
-	if (www_auth!=NULL)
-		www_realm=osip_authorization_get_realm(www_auth);
-	
-	if (prx_realm==NULL && www_realm==NULL){
-		ms_message("No authentication info in the request, ignoring");
-		return;
-	}
-	/* see if we already have this auth information , not to ask it everytime to the user */
-	if (prx_realm!=NULL)
-		as=linphone_core_find_auth_info(lc,prx_realm,username);
-	if (www_realm!=NULL) 
-		as=linphone_core_find_auth_info(lc,www_realm,username);
-	if (as){
-		ms_message("Authentication for user=%s realm=%s is working.",username,prx_realm ? prx_realm : www_realm);
-		as->works=TRUE;
-	}
-}
-
-
-void linphone_core_find_or_ask_for_auth_info(LinphoneCore *lc,const char *username,const char* realm, int tid)
-{
-	LinphoneAuthInfo *as=linphone_core_find_auth_info(lc,realm,username);
-	if ( as==NULL || (as!=NULL && as->works==FALSE && as->first_time==FALSE)){
-		if (lc->vtable.auth_info_requested!=NULL){
-			lc->vtable.auth_info_requested(lc,realm,username);
-			lc->automatic_action++;/*suspends eXosip_automatic_action until the user supplies a password */
-		}
-	}
-	if (as) as->first_time=FALSE;
-}
-
-void linphone_process_authentication(LinphoneCore *lc, eXosip_event_t *ev)
-{
-	char *prx_realm=NULL,*www_realm=NULL;
-	osip_proxy_authenticate_t *prx_auth;
-	osip_www_authenticate_t *www_auth;
-	osip_message_t *resp=ev->response;
-	char *username;
-	
-	if (strcmp(ev->request->sip_method,"REGISTER")==0) {
-		gstate_new_state(lc, GSTATE_REG_FAILED, "Authentication required");
-	}
-
-	username=osip_uri_get_username(resp->from->url);
-	prx_auth=(osip_proxy_authenticate_t*)osip_list_get(&resp->proxy_authenticates,0);
-	www_auth=(osip_proxy_authenticate_t*)osip_list_get(&resp->www_authenticates,0);
-	if (prx_auth!=NULL)
-		prx_realm=osip_proxy_authenticate_get_realm(prx_auth);
-	if (www_auth!=NULL)
-		www_realm=osip_www_authenticate_get_realm(www_auth);
-	
-	if (prx_realm==NULL && www_realm==NULL){
-		ms_warning("No realm in the server response.");
-		return;
-	}
-	/* see if we already have this auth information , not to ask it everytime to the user */
-	if (prx_realm!=NULL) 
-		linphone_core_find_or_ask_for_auth_info(lc,username,prx_realm,ev->tid);
-	if (www_realm!=NULL) 
-		linphone_core_find_or_ask_for_auth_info(lc,username,www_realm,ev->tid);
-}
-
+/**
+ * @}
+**/
