@@ -24,18 +24,16 @@
  
  #include "linphonecore.h"
  #include "private.h"
- #include <eXosip2/eXosip.h>
  
  LinphoneChatRoom * linphone_core_create_chat_room(LinphoneCore *lc, const char *to){
-	char *real_url=NULL;
-	osip_from_t *parsed_url=NULL;
-	char *route;
-	if (linphone_core_interpret_url(lc,to,&real_url,&parsed_url,&route)){
+	LinphoneAddress *parsed_url=NULL;
+
+	if ((parsed_url=linphone_core_interpret_url(lc,to))!=NULL){
 		LinphoneChatRoom *cr=ms_new0(LinphoneChatRoom,1);
 		cr->lc=lc;
-		cr->peer=real_url;
+		cr->peer=linphone_address_as_string(parsed_url);
 		cr->peer_url=parsed_url;
-		cr->route=route;
+		cr->route=ms_strdup(linphone_core_get_route(lc));
 		lc->chatrooms=ms_list_append(lc->chatrooms,(void *)cr);
 		return cr;
 	}
@@ -46,23 +44,30 @@
  void linphone_chat_room_destroy(LinphoneChatRoom *cr){
 	LinphoneCore *lc=cr->lc;
 	lc->chatrooms=ms_list_remove(lc->chatrooms,(void *) cr);
-	osip_from_free(cr->peer_url);
+	linphone_address_destroy(cr->peer_url);
 	ms_free(cr->peer);
 	ms_free(cr->route);
  }
  
 void linphone_chat_room_send_message(LinphoneChatRoom *cr, const char *msg){
 	const char *identity=linphone_core_get_identity(cr->lc);
-	osip_message_t *sip=NULL;
-	eXosip_message_build_request(&sip,"MESSAGE",cr->peer,identity,cr->route);
-	osip_message_set_content_type(sip,"text/plain");
-	osip_message_set_body(sip,msg,strlen(msg));
-	eXosip_message_send_request(sip);
+	SalOp *op;
+	if(linphone_core_is_in_communication_with(cr->lc,cr->peer))
+	{
+		ms_message("send SIP message into the call\n");
+		op = cr->lc->call->op;
+	}
+	else
+	{
+		op = sal_op_new(cr->lc->sal);
+		sal_op_set_route(op,cr->route);
+	}
+	sal_text_send(op,identity,cr->peer,msg);
 }
 
-bool_t linphone_chat_room_matches(LinphoneChatRoom *cr, osip_from_t *from){
-	if (cr->peer_url->url->username && from->url->username && 
-		strcmp(cr->peer_url->url->username,from->url->username)==0) return TRUE;
+bool_t linphone_chat_room_matches(LinphoneChatRoom *cr, const LinphoneAddress *from){
+	if (linphone_address_get_username(cr->peer_url) && linphone_address_get_username(from) && 
+		strcmp(linphone_address_get_username(cr->peer_url),linphone_address_get_username(from))==0) return TRUE;
 	return FALSE;
 }
 
@@ -70,34 +75,29 @@ void linphone_chat_room_text_received(LinphoneChatRoom *cr, LinphoneCore *lc, co
 	if (lc->vtable.text_received!=NULL) lc->vtable.text_received(lc, cr, from, msg);
 }
 
-void linphone_core_text_received(LinphoneCore *lc, eXosip_event_t *ev){
+void linphone_core_text_received(LinphoneCore *lc, const char *from, const char *msg){
 	MSList *elem;
-	const char *msg;
 	LinphoneChatRoom *cr=NULL;
+	LinphoneAddress *addr;
 	char *cleanfrom;
-	osip_from_t *from_url=ev->request->from;
-	osip_body_t *body=NULL;
 
-	osip_message_get_body(ev->request,0,&body);
-	if (body==NULL){
-		ms_error("Could not get text message from SIP body");
-		return;
-	}
-	msg=body->body;
-	from_2char_without_params(from_url,&cleanfrom);
+	addr=linphone_address_new(from);
+	linphone_address_clean(addr);
 	for(elem=lc->chatrooms;elem!=NULL;elem=ms_list_next(elem)){
 		cr=(LinphoneChatRoom*)elem->data;
-		if (linphone_chat_room_matches(cr,from_url)){
+		if (linphone_chat_room_matches(cr,addr)){
 			break;
 		}
 		cr=NULL;
 	}
+	cleanfrom=linphone_address_as_string(addr);
 	if (cr==NULL){
 		/* create a new chat room */
 		cr=linphone_core_create_chat_room(lc,cleanfrom);
 	}
+	linphone_address_destroy(addr);
 	linphone_chat_room_text_received(cr,lc,cleanfrom,msg);
-	osip_free(cleanfrom);
+	ms_free(cleanfrom);
 }
 
 
