@@ -44,6 +44,15 @@ struct SalOp;
 struct _LpConfig;
 
 
+struct _LCSipTransports{
+	int udp_port;
+	int tcp_port;
+	int dtls_port;
+	int tls_port;
+};
+
+typedef struct _LCSipTransports LCSipTransports;
+
 /**
  * Object that represents a SIP address.
  *
@@ -66,6 +75,15 @@ const char *linphone_address_get_scheme(const LinphoneAddress *u);
 const char *linphone_address_get_display_name(const LinphoneAddress* u);
 const char *linphone_address_get_username(const LinphoneAddress *u);
 const char *linphone_address_get_domain(const LinphoneAddress *u);
+/**
+ * Get port number as an integer value.
+ *
+ */
+int linphone_address_get_port_int(const LinphoneAddress *u);
+/**
+ * Get port number, null if not present.
+ */
+const char* linphone_address_get_port(const LinphoneAddress *u);
 void linphone_address_set_display_name(LinphoneAddress *u, const char *display_name);
 void linphone_address_set_username(LinphoneAddress *uri, const char *username);
 void linphone_address_set_domain(LinphoneAddress *uri, const char *host);
@@ -555,6 +573,14 @@ bool_t linphone_core_payload_type_enabled(LinphoneCore *lc, PayloadType *pt);
 
 int linphone_core_enable_payload_type(LinphoneCore *lc, PayloadType *pt, bool_t enable);
 
+/*
+ * get payload type  from mime type an clock rate
+ * @ingroup media_parameters
+ * iterates both audio an video
+ * return NULL if not found
+ */
+PayloadType* linphone_core_find_payload_type(LinphoneCore* lc, const char* type, int rate) ;
+
 const char *linphone_core_get_payload_type_description(LinphoneCore *lc, PayloadType *pt);
 
 bool_t linphone_core_check_payload_type_usability(LinphoneCore *lc, PayloadType *pt);
@@ -609,9 +635,13 @@ void linphone_core_set_use_rfc2833_for_dtmf(LinphoneCore *lc,bool_t use_rfc2833)
 
 bool_t linphone_core_get_use_rfc2833_for_dtmf(LinphoneCore *lc);
 
+void linphone_core_set_sip_port(LinphoneCore *lc, int port);
+
 int linphone_core_get_sip_port(LinphoneCore *lc);
 
-void linphone_core_set_sip_port(LinphoneCore *lc,int port);
+int linphone_core_set_sip_transports(LinphoneCore *lc, const LCSipTransports *transports);
+
+int linphone_core_get_sip_transports(LinphoneCore *lc, LCSipTransports *transports);
 
 ortp_socket_t linphone_core_get_sip_socket(LinphoneCore *lc);
 
@@ -645,18 +675,10 @@ int linphone_core_get_play_level(LinphoneCore *lc);
 int linphone_core_get_rec_level(LinphoneCore *lc);
 void linphone_core_set_ring_level(LinphoneCore *lc, int level);
 void linphone_core_set_play_level(LinphoneCore *lc, int level);
-/**
- * Allow to control play level before entering  sound card:  level in db
- *
- * @ingroup media_parameters
-**/
-void linphone_core_set_soft_play_level(LinphoneCore *lc, float level);
-/**
- * get play level before entering  sound card:  level in db
- *
- * @ingroup media_parameters
-**/
-float linphone_core_get_soft_play_level(LinphoneCore *lc);
+
+void linphone_core_set_playback_gain_db(LinphoneCore *lc, float level);
+
+float linphone_core_get_playback_gain_db(LinphoneCore *lc);
 void linphone_core_set_rec_level(LinphoneCore *lc, int level);
 const char * linphone_core_get_ringer_device(LinphoneCore *lc);
 const char * linphone_core_get_playback_device(LinphoneCore *lc);
@@ -688,6 +710,12 @@ void linphone_core_mute_mic(LinphoneCore *lc, bool_t muted);
 **/
 bool_t linphone_core_is_mic_muted(LinphoneCore *lc);
 
+bool_t linphone_core_is_audio_muted(LinphoneCore *lc);
+bool_t linphone_core_is_rtp_muted(LinphoneCore *lc);
+
+bool_t linphone_core_get_rtp_no_xmit_on_audio_mute(const LinphoneCore *lc);
+void linphone_core_set_rtp_no_xmit_on_audio_mute(LinphoneCore *lc, bool_t val);
+
 void linphone_core_set_presence_info(LinphoneCore *lc,int minutes_away,const char *contact,LinphoneOnlineStatus os);
 
 LinphoneOnlineStatus linphone_core_get_presence_info(const LinphoneCore *lc);
