@@ -161,9 +161,11 @@ extern "C" jlong Java_org_linphone_core_LinphoneCoreImpl_newLinphoneCore(JNIEnv*
 	LinphoneCoreData* ldata = new LinphoneCoreData(env,thiz,jlistener,juserdata);
 #ifdef ANDROID
 	ms_andsnd_register_card(jvm);
-	// requires an fpu libmsilbc_init();
 #endif /*ANDROID*/
 
+#ifdef HAVE_ILBC
+	libmsilbc_init(); // requires an fpu
+#endif
 	jlong nativePtr = (jlong)linphone_core_new(	&ldata->vTable
 			,userConfig
 			,factoryConfig
@@ -292,17 +294,17 @@ extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setNetworkStateReachable
 		linphone_core_set_network_reachable((LinphoneCore*)lc,isReachable);
 }
 
-extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setSoftPlayLevel(	JNIEnv*  env
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setPlaybackGain(	JNIEnv*  env
 		,jobject  thiz
 		,jlong lc
 		,jfloat gain) {
-		linphone_core_set_soft_play_level((LinphoneCore*)lc,gain);
+		linphone_core_set_playback_gain_db((LinphoneCore*)lc,gain);
 }
 
-extern "C" float Java_org_linphone_core_LinphoneCoreImpl_getSoftPlayLevel(	JNIEnv*  env
+extern "C" float Java_org_linphone_core_LinphoneCoreImpl_getPlaybackGain(	JNIEnv*  env
 		,jobject  thiz
 		,jlong lc) {
-		return linphone_core_get_soft_play_level((LinphoneCore*)lc);
+		return linphone_core_get_playback_gain_db((LinphoneCore*)lc);
 }
 
 extern "C" void Java_org_linphone_core_LinphoneCoreImpl_muteMic(	JNIEnv*  env
@@ -337,7 +339,35 @@ extern "C" jboolean Java_org_linphone_core_LinphoneCoreImpl_isMicMuted(	JNIEnv*
 		,jlong lc) {
 	return linphone_core_is_mic_muted((LinphoneCore*)lc);
 }
-
+extern "C" jlong Java_org_linphone_core_LinphoneCoreImpl_findPayloadType(JNIEnv*  env
+																			,jobject  thiz
+																			,jlong lc
+																			,jstring jmime
+																			,jint rate) {
+	const char* mime = env->GetStringUTFChars(jmime, NULL);
+	jlong result = (jlong)linphone_core_find_payload_type((LinphoneCore*)lc,mime,rate);
+	env->ReleaseStringUTFChars(jmime, mime);
+	return result;
+}
+extern "C" jlong Java_org_linphone_core_LinphoneCoreImpl_enablePayloadType(JNIEnv*  env
+																			,jobject  thiz
+																			,jlong lc
+																			,jlong pt
+																			,jboolean enable) {
+	return linphone_core_enable_payload_type((LinphoneCore*)lc,(PayloadType*)pt,enable);
+}
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_enableEchoCancellation(JNIEnv*  env
+																			,jobject  thiz
+																			,jlong lc
+																			,jboolean enable) {
+	linphone_core_enable_echo_cancellation((LinphoneCore*)lc,enable);
+}
+extern "C" jboolean Java_org_linphone_core_LinphoneCoreImpl_isEchoCancellationEnabled(JNIEnv*  env
+																			,jobject  thiz
+																			,jlong lc
+																			) {
+	return linphone_core_echo_cancellation_enabled((LinphoneCore*)lc);
+}
 
 
 //ProxyConfig
@@ -377,6 +407,20 @@ extern "C" jstring Java_org_linphone_core_LinphoneProxyConfigImpl_getProxy(JNIEn
 		return NULL;
 	}
 }
+extern "C" int Java_org_linphone_core_LinphoneProxyConfigImpl_setRoute(JNIEnv* env,jobject thiz,jlong proxyCfg,jstring jroute) {
+	const char* route = env->GetStringUTFChars(jroute, NULL);
+	int err=linphone_proxy_config_set_route((LinphoneProxyConfig*)proxyCfg,route);
+	env->ReleaseStringUTFChars(jroute, route);
+	return err;
+}
+extern "C" jstring Java_org_linphone_core_LinphoneProxyConfigImpl_getRoute(JNIEnv* env,jobject thiz,jlong proxyCfg) {
+	const char* route = linphone_proxy_config_get_route((LinphoneProxyConfig*)proxyCfg);
+	if (route) {
+		return env->NewStringUTF(route);
+	} else {
+		return NULL;
+	}
+}
 
 extern "C" void Java_org_linphone_core_LinphoneProxyConfigImpl_enableRegister(JNIEnv* env,jobject thiz,jlong proxyCfg,jboolean enableRegister) {
 	linphone_proxy_config_enable_register((LinphoneProxyConfig*)proxyCfg,enableRegister);
@@ -531,9 +575,9 @@ extern "C" void Java_org_linphone_core_LinphoneAddressImpl_setDisplayName(JNIEnv
 																		,jobject  thiz
 																		,jlong address
 																		,jstring jdisplayName) {
-	const char* displayName = env->GetStringUTFChars(jdisplayName, NULL);
+	const char* displayName = jdisplayName!= NULL?env->GetStringUTFChars(jdisplayName, NULL):NULL;
 	linphone_address_set_display_name((LinphoneAddress*)address,displayName);
-	env->ReleaseStringUTFChars(jdisplayName, displayName);
+	if (displayName != NULL) env->ReleaseStringUTFChars(jdisplayName, displayName);
 }
 
 
@@ -553,3 +597,17 @@ extern "C" jboolean Java_org_linphone_core_LinphoneCallLogImpl_isIncoming(JNIEnv
 																		,jlong ptr) {
 	return ((LinphoneCallLog*)ptr)->dir==LinphoneCallIncoming?JNI_TRUE:JNI_FALSE;
 }
+
+extern "C" jstring Java_org_linphone_core_PayloadTypeImpl_toString(JNIEnv*  env
+																		,jobject  thiz
+																		,jlong ptr) {
+
+	PayloadType* pt = (PayloadType*)ptr;
+	char* value = ms_strdup_printf("[%s] clock [%s], bitrate [%s]"
+									,payload_type_get_mime(pt)
+									,payload_type_get_rate(pt)
+									,payload_type_get_bitrate(pt));
+	jstring jvalue =env->NewStringUTF(value);
+	ms_free(value);
+	return jvalue;
+}
