@@ -55,15 +55,16 @@ static gboolean do_login_noprompt(LinphoneProxyConfig *cfg){
 void linphone_gtk_show_login_frame(LinphoneProxyConfig *cfg){
 	GtkWidget *mw=linphone_gtk_get_main_window();
 	GtkWidget *label=linphone_gtk_get_widget(mw,"login_label");
-	LinphoneAuthInfo *ai;
+	const LinphoneAuthInfo *ai;
 	gchar *str;
-	osip_from_t *from;
+	LinphoneAddress *from;
 	LinphoneCore *lc=linphone_gtk_get_core();
 	int nettype;
+	const char *passwd=NULL;
 
 
-	if (linphone_core_get_download_bandwidth(lc)==0 &&
-		linphone_core_get_upload_bandwidth(lc)==0)
+	if (linphone_core_get_download_bandwidth(lc)==512 &&
+		linphone_core_get_upload_bandwidth(lc)==512)
 		nettype=NetworkKindOpticalFiber;
 	else nettype=NetworkKindAdsl;
 	gtk_combo_box_set_active(GTK_COMBO_BOX(linphone_gtk_get_widget(mw,"login_internet_kind")),nettype);
@@ -84,18 +85,18 @@ void linphone_gtk_show_login_frame(LinphoneProxyConfig *cfg){
 	g_object_set_data(G_OBJECT(mw),"login_proxy_config",cfg);
 	g_free(str);
 
-	osip_from_init(&from);
-	osip_from_parse(from,linphone_proxy_config_get_identity(cfg));
+	from=linphone_address_new(linphone_proxy_config_get_identity(cfg));
 	
-	ai=linphone_core_find_auth_info(lc,linphone_proxy_config_get_domain(cfg),from->url->username);
+	ai=linphone_core_find_auth_info(lc,linphone_proxy_config_get_domain(cfg),linphone_address_get_username(from));
 	/*display the last entered username, if not '?????'*/
-	if (from->url->username[0]!='?')
+	if (linphone_address_get_username(from)[0]!='?')
 		gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(mw,"login_username")),
-			from->url->username);
+			linphone_address_get_username(from));
+	if (ai) passwd=linphone_auth_info_get_passwd(ai);
 	gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(mw,"login_password")),
-		ai!=NULL ? ai->passwd : "");
+		passwd!=NULL ? passwd : "");
 	
-	osip_from_free(from);
+	linphone_address_destroy(from);
 }
 
 void linphone_gtk_exit_login_frame(void){
@@ -130,21 +131,21 @@ void linphone_gtk_login_frame_connect_clicked(GtkWidget *button){
 	char *identity;
 	gboolean autologin;
 	LinphoneProxyConfig *cfg=(LinphoneProxyConfig*)g_object_get_data(G_OBJECT(mw),"login_proxy_config");
-	osip_from_t *from;
+	LinphoneAddress *from;
 	SipSetupContext *ssctx=linphone_proxy_config_get_sip_setup_context(cfg);
 
 	username=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(mw,"login_username")));
 	password=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(mw,"login_password")));
 
+	if (username==NULL || username[0]=='\0')
+		return;
+
 	autologin=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(mw,"automatic_login")));
 	linphone_gtk_set_ui_config_int("automatic_login",autologin);
 
-	osip_from_init(&from);
-	osip_from_parse(from,linphone_proxy_config_get_identity(cfg));
-	osip_free(from->url->username);
-	from->url->username=osip_strdup(username);
-	osip_from_to_str(from,&identity);
-	osip_from_free(from);
+	from=linphone_address_new(linphone_proxy_config_get_identity(cfg));
+	linphone_address_set_username(from,username);
+	identity=linphone_address_as_string(from);
 	do_login(ssctx,identity,password);
 	/*we need to refresh the identities since the proxy config may have changed.*/
 	linphone_gtk_load_identities();
