@@ -19,6 +19,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #define USE_LIBGLADE 1
 
+#define VIDEOSELFVIEW_DEFAULT 1
+
 #include "linphone.h"
 #include "lpconfig.h"
 
@@ -32,7 +34,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <sys/stat.h>
 #include <unistd.h>
 
-#define LINPHONE_ICON "linphone2.png"
+#define LINPHONE_ICON "linphone.png"
 
 const char *this_program_ident_string="linphone_ident_string=" LINPHONE_VERSION;
 
@@ -42,7 +44,7 @@ static GtkWidget *the_ui=NULL;
 static void linphone_gtk_show(LinphoneCore *lc);
 static void linphone_gtk_inv_recv(LinphoneCore *lc, const char *from);
 static void linphone_gtk_bye_recv(LinphoneCore *lc, const char *from);
-static void linphone_gtk_notify_recv(LinphoneCore *lc, LinphoneFriend * fid, const char *url, const char *status, const char *img);
+static void linphone_gtk_notify_recv(LinphoneCore *lc, LinphoneFriend * fid);
 static void linphone_gtk_new_unknown_subscriber(LinphoneCore *lc, LinphoneFriend *lf, const char *url);
 static void linphone_gtk_auth_info_requested(LinphoneCore *lc, const char *realm, const char *username);
 static void linphone_gtk_display_status(LinphoneCore *lc, const char *status);
@@ -59,7 +61,7 @@ static LinphoneCoreVTable vtable={
 	.show=linphone_gtk_show,
 	.inv_recv=linphone_gtk_inv_recv,
 	.bye_recv=linphone_gtk_bye_recv,
-	.notify_recv=linphone_gtk_notify_recv,
+	.notify_presence_recv=linphone_gtk_notify_recv,
 	.new_unknown_subscriber=linphone_gtk_new_unknown_subscriber,
 	.auth_info_requested=linphone_gtk_auth_info_requested,
 	.display_status=linphone_gtk_display_status,
@@ -70,7 +72,8 @@ static LinphoneCoreVTable vtable={
 	.call_log_updated=linphone_gtk_call_log_updated,
 	.text_received=linphone_gtk_text_received,
 	.general_state=linphone_gtk_general_state,
-	.refer_received=linphone_gtk_refer_received
+	.refer_received=linphone_gtk_refer_received,
+	.buddy_info_updated=linphone_gtk_buddy_info_updated
 };
 
 static gboolean verbose=0;
@@ -80,6 +83,7 @@ static gboolean iconified=FALSE;
 #ifdef WIN32
 static gchar *workingdir=NULL;
 #endif
+static char *progpath=NULL;
 
 static GOptionEntry linphone_options[]={
 	{
@@ -157,9 +161,58 @@ const char *linphone_gtk_get_config_file(){
 	return _config_file;
 }
 
-static void linphone_gtk_init_liblinphone(const char *file){
+
+#define FACTORY_CONFIG_FILE "linphonerc.factory"
+static char _factory_config_file[1024];
+static const char *linphone_gtk_get_factory_config_file(){
+	/*try accessing a local file first if exists*/
+	if (access(FACTORY_CONFIG_FILE,F_OK)==0){
+		snprintf(_factory_config_file,sizeof(_factory_config_file),
+						 "%s",FACTORY_CONFIG_FILE);
+	} else {
+		char *progdir;
+		
+		if (progpath != NULL) {
+			char *basename;
+			progdir = strdup(progpath);
+#ifdef WIN32
+			basename = strrchr(progdir, '\\');
+			if (basename != NULL) {
+				basename ++;
+				*basename = '\0';
+				snprintf(_factory_config_file, sizeof(_factory_config_file),
+								 "%s\\..\\%s", progdir, FACTORY_CONFIG_FILE);
+			} else {
+				if (workingdir!=NULL) {
+					snprintf(_factory_config_file, sizeof(_factory_config_file),
+									 "%s\\%s", workingdir, FACTORY_CONFIG_FILE);
+				} else {
+					free(progdir);
+					return NULL;
+				}
+			}
+#else
+			basename = strrchr(progdir, '/');
+			if (basename != NULL) {
+				basename ++;
+				*basename = '\0';
+				snprintf(_factory_config_file, sizeof(_factory_config_file),
+								 "%s/../share/Linphone/%s", progdir, FACTORY_CONFIG_FILE);
+			} else {
+				free(progdir);
+				return NULL;
+			}
+#endif
+			free(progdir);
+		}
+	}
+	return _factory_config_file;
+}
+
+static void linphone_gtk_init_liblinphone(const char *config_file,
+		const char *factory_config_file) {
 	linphone_core_set_user_agent("Linphone", LINPHONE_VERSION);
-	the_core=linphone_core_new(&vtable,file,NULL);
+	the_core=linphone_core_new(&vtable,config_file,factory_config_file,NULL);
 	linphone_core_set_waiting_callback(the_core,linphone_gtk_wait,NULL);
 }
 
@@ -173,46 +226,6 @@ GtkWidget *linphone_gtk_get_main_window(){
 	return the_ui;
 }
 
-static void parse_item(const char *item, const char *window_name, GtkWidget *w,  gboolean show){
-	char tmp[64];
-	char *dot;
-	strcpy(tmp,item);
-	dot=strchr(tmp,'.');
-	if (dot){
-		*dot='\0';
-		dot++;
-		if (strcmp(window_name,tmp)==0){
-			GtkWidget *wd=linphone_gtk_get_widget(w,dot);
-			if (wd) {
-				if (!show) gtk_widget_hide(wd);
-				else gtk_widget_show(wd);
-			}
-		}
-	}
-}
-
-static void parse_widgets(const char *hiddens, const char *window_name, GtkWidget *w, gboolean show){
-	char item[64];
-	const char *i;
-	const char *b;
-	int len;
-	for(b=i=hiddens;*i!='\0';++i){
-		if (*i==' '){
-			len=MIN(i-b,sizeof(item)-1);
-			strncpy(item,b,len);
-			item[len]='\0';
-			b=i+1;
-			parse_item(item,window_name,w,show);
-		}
-	}
-	len=MIN(i-b,sizeof(item)-1);
-	if (len>0){
-		strncpy(item,b,len);
-		item[len]='\0';
-		parse_item(item,window_name,w,show);
-	}
-}
-
 static void linphone_gtk_configure_window(GtkWidget *w, const char *window_name){
 	static const char *icon_path=NULL;
 	static const char *hiddens=NULL;
@@ -222,13 +235,13 @@ static void linphone_gtk_configure_window(GtkWidget *w, const char *window_name)
 	if (config_loaded==FALSE){
 		hiddens=linphone_gtk_get_ui_config("hidden_widgets",NULL);
 		shown=linphone_gtk_get_ui_config("shown_widgets",NULL);
-		icon_path=linphone_gtk_get_ui_config("icon",NULL);
+		icon_path=linphone_gtk_get_ui_config("icon",LINPHONE_ICON);
 		config_loaded=TRUE;
 	}
 	if (hiddens)
-		parse_widgets(hiddens,window_name,w,FALSE);
+		linphone_gtk_visibility_set(hiddens,window_name,w,FALSE);
 	if (shown)
-		parse_widgets(shown,window_name,w,TRUE);
+		linphone_gtk_visibility_set(shown,window_name,w,TRUE);
 	if (icon_path) {
 		GdkPixbuf *pbuf=create_pixbuf(icon_path);
 		gtk_window_set_icon(GTK_WINDOW(w),pbuf);
@@ -339,6 +352,8 @@ void linphone_gtk_show_about(){
 	struct stat filestat;
 	const char *license_file=PACKAGE_DATA_DIR "/linphone/COPYING";
 	GtkWidget *about;
+	GdkPixbuf *logo=create_pixbuf(
+	    linphone_gtk_get_ui_config("logo","linphone-banner.png"));
 	
 	about=linphone_gtk_create_window("about");
 	gtk_about_dialog_set_url_hook(about_url_clicked,NULL,NULL);
@@ -359,25 +374,48 @@ void linphone_gtk_show_about(){
 	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about),LINPHONE_VERSION);
 	gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(about),linphone_gtk_get_ui_config("title","Linphone"));
 	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(about),linphone_gtk_get_ui_config("home","http://www.linphone.org"));
+	if (logo)	gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(about),logo);
+	    
 	gtk_widget_show(about);
 }
 
 static void set_video_window_decorations(GdkWindow *w){
 	const char *title=linphone_gtk_get_ui_config("title","Linphone");
-	const char *icon_path=linphone_gtk_get_ui_config("icon","linphone2.png");
+	const char *icon_path=linphone_gtk_get_ui_config("icon",LINPHONE_ICON);
 	char video_title[256];
 	GdkPixbuf *pbuf=create_pixbuf(icon_path);
 	if (!linphone_core_in_call(linphone_gtk_get_core())){
-		snprintf(video_title,sizeof(video_title),"%s video",title);	
+		snprintf(video_title,sizeof(video_title),"%s video",title);
+		/* When not in call, treat the video as a normal window */
+		gdk_window_set_keep_above(w, FALSE);
 	}else{
-		const char *uri=linphone_core_get_remote_uri(linphone_gtk_get_core());
-		gchar *display_name=linphone_gtk_get_display_name(uri);
-		snprintf(video_title,sizeof(video_title),"Call with %s",display_name);
-		g_free(display_name);
+		LinphoneAddress *uri =
+			linphone_address_clone(linphone_core_get_remote_uri(linphone_gtk_get_core()));
+		char *display_name;
+
+		linphone_address_clean(uri);
+		if (linphone_address_get_display_name(uri)!=NULL){
+			display_name=ms_strdup(linphone_address_get_display_name(uri));
+		}else{
+			display_name=linphone_address_as_string(uri);
+		}
+		snprintf(video_title,sizeof(video_title),_("Call with %s"),display_name);
+		linphone_address_destroy(uri);
+		ms_free(display_name);
+
+		/* During calls, bring up the video window, arrange so that
+		it is above all the other windows */
+		gdk_window_deiconify(w);
+		gdk_window_set_keep_above(w,TRUE);
+		/* Maybe we should have the following, but then we want to
+		have a timer that turns it off after a little while. */
+		/* gdk_window_set_urgency_hint(w,TRUE); */
 	}
 	gdk_window_set_title(w,video_title);
-	/*gdk_window_set_urgency_hint(w,TRUE);*/
-	gdk_window_raise(w);
+	/* Refrain the video window to be closed at all times. */
+	gdk_window_set_functions(w,
+				 GDK_FUNC_RESIZE|GDK_FUNC_MOVE|
+				 GDK_FUNC_MINIMIZE|GDK_FUNC_MAXIMIZE);
 	if (pbuf){
 		GList *l=NULL;
 		l=g_list_append(l,pbuf);
@@ -520,7 +558,6 @@ void linphone_gtk_call_terminated(const char *error){
 	GtkWidget *icw;
 	gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"terminate_call"),FALSE);
 	gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"start_call"),TRUE);
-	gtk_widget_hide_all(linphone_gtk_get_widget(mw,"go_to_call_view_box"));
 	linphone_gtk_enable_mute_button(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(mw,"main_mute")),FALSE);
 	if (linphone_gtk_use_in_call_view())
 		linphone_gtk_in_call_view_terminate(error);
@@ -544,7 +581,6 @@ static gboolean in_call_timer(){
 static void linphone_gtk_call_started(GtkWidget *mw){
 	gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"start_call"),FALSE);
 	gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"terminate_call"),TRUE);
-	gtk_widget_show_all(linphone_gtk_get_widget(mw,"go_to_call_view_box"));
 	update_video_title();
 	if (linphone_gtk_use_in_call_view())
 		g_timeout_add(250,(GSourceFunc)in_call_timer,NULL);
@@ -555,6 +591,7 @@ static gboolean linphone_gtk_start_call_do(GtkWidget *uri_bar){
 	if (linphone_core_invite(linphone_gtk_get_core(),entered)==0) {
 		completion_add_text(GTK_ENTRY(uri_bar),entered);
 	}else{
+		linphone_gtk_call_terminated(NULL);
 	}
 	return FALSE;
 }
@@ -611,9 +648,6 @@ void linphone_gtk_terminate_call(GtkWidget *button){
 
 void linphone_gtk_decline_call(GtkWidget *button){
 	linphone_core_terminate_call(linphone_gtk_get_core(),NULL);
-	/* zsd note: there was a big here in 3.0.0 which caused an abort if
-	 * someone clicked "decline"... the following line of code looks
-	 * like a fix for that. */
 	gtk_widget_destroy(gtk_widget_get_toplevel(button));
 }
 
@@ -628,7 +662,8 @@ static gboolean linphone_gtk_auto_answer(GtkWidget *incall_window){
 
 void linphone_gtk_set_audio_video(){
 	linphone_core_enable_video(linphone_gtk_get_core(),TRUE,TRUE);
-	linphone_core_enable_video_preview(linphone_gtk_get_core(),TRUE);
+	linphone_core_enable_video_preview(linphone_gtk_get_core(),
+	    linphone_gtk_get_ui_config_int("videoselfview",VIDEOSELFVIEW_DEFAULT));
 }
 
 void linphone_gtk_set_audio_only(){
@@ -637,8 +672,11 @@ void linphone_gtk_set_audio_only(){
 }
 
 void linphone_gtk_enable_self_view(GtkWidget *w){
-	linphone_core_enable_self_view(linphone_gtk_get_core(),
-		gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(w)));
+	gboolean val=gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(w));
+	LinphoneCore *lc=linphone_gtk_get_core();
+	linphone_core_enable_video_preview(lc,val);
+	linphone_core_enable_self_view(lc,val);
+	linphone_gtk_set_ui_config_int("videoselfview",val);
 }
 
 void linphone_gtk_used_identity_changed(GtkWidget *w){
@@ -648,12 +686,16 @@ void linphone_gtk_used_identity_changed(GtkWidget *w){
 		linphone_core_set_default_proxy_index(linphone_gtk_get_core(),(active==0) ? -1 : (active-1));
 		linphone_gtk_show_directory_search();
 	}
+	if (sel) g_free(sel);
 }
 
 static void linphone_gtk_show_main_window(){
 	GtkWidget *w=linphone_gtk_get_main_window();
 	LinphoneCore *lc=linphone_gtk_get_core();
-	linphone_core_enable_video_preview(lc,linphone_core_video_enabled(lc));
+	if (linphone_core_video_enabled(lc)){
+		linphone_core_enable_video_preview(lc,linphone_gtk_get_ui_config_int("videoselfview",
+	    	VIDEOSELFVIEW_DEFAULT));
+	}
 	gtk_widget_show(w);
 	gtk_window_present(GTK_WINDOW(w));
 }
@@ -691,7 +733,8 @@ static void linphone_gtk_bye_recv(LinphoneCore *lc, const char *from){
 	
 }
 
-static void linphone_gtk_notify_recv(LinphoneCore *lc, LinphoneFriend * fid, const char *url, const char *status, const char *img){
+static void linphone_gtk_notify_recv(LinphoneCore *lc, LinphoneFriend * fid){
+	linphone_gtk_show_friends();
 }
 
 static void linphone_gtk_new_subscriber_response(GtkWidget *dialog, guint response_id, LinphoneFriend *lf){
@@ -707,6 +750,12 @@ static void linphone_gtk_new_subscriber_response(GtkWidget *dialog, guint respon
 
 static void linphone_gtk_new_unknown_subscriber(LinphoneCore *lc, LinphoneFriend *lf, const char *url){
 	GtkWidget *dialog;
+
+	if (linphone_gtk_get_ui_config_int("subscribe_deny_all",0)){
+		linphone_core_reject_subscriber(linphone_gtk_get_core(),lf);
+		return;
+	}
+
 	gchar *message=g_strdup_printf(_("%s would like to add you to his contact list.\nWould you allow him to see your presence status or add him to your contact list ?\nIf you answer no, this person will be temporarily blacklisted."),url);
 	dialog = gtk_message_dialog_new (
 				GTK_WINDOW(linphone_gtk_get_main_window()),
@@ -777,6 +826,14 @@ static void linphone_gtk_auth_info_requested(LinphoneCore *lc, const char *realm
 	GtkWidget *label=linphone_gtk_get_widget(w,"message");
 	LinphoneAuthInfo *info;
 	gchar *msg;
+	GtkWidget *mw=linphone_gtk_get_main_window();
+	
+	if (mw && GTK_WIDGET_VISIBLE(linphone_gtk_get_widget(mw,"login_frame"))){
+		/*don't prompt for authentication when login frame is visible*/
+		linphone_core_abort_authentication(lc,NULL);
+		return;
+	}
+
 	msg=g_strdup_printf(_("Please enter your password for username <i>%s</i>\n at domain <i>%s</i>:"),
 		username,realm);
 	gtk_label_set_markup(GTK_LABEL(label),msg);
@@ -848,14 +905,17 @@ static void icon_popup_menu(GtkStatusIcon *status_icon, guint button, guint acti
 
 void linphone_gtk_open_browser(const char *url){
 	/*in gtk 2.16, gtk_show_uri does not work...*/
-	/*gtk_show_uri(NULL,url,GDK_CURRENT_TIME,NULL);*/
-#ifdef WIN32
-	ShellExecute(0,"open",url,NULL,NULL,1);
+#ifndef WIN32
+#if GTK_CHECK_VERSION(2,18,3)
+	gtk_show_uri(NULL,url,GDK_CURRENT_TIME,NULL);
 #else
 	char cl[255];
 	snprintf(cl,sizeof(cl),"/usr/bin/x-www-browser %s",url);
 	g_spawn_command_line_async(cl,NULL);
 #endif
+#else /*WIN32*/
+	ShellExecute(0,"open",url,NULL,NULL,1);
+#endif
 }
 
 void linphone_gtk_link_to_website(GtkWidget *item){
@@ -964,9 +1024,12 @@ static void linphone_gtk_connect_digits(void){
 
 static void linphone_gtk_check_menu_items(void){
 	bool_t audio_only=!linphone_core_video_enabled(linphone_gtk_get_core());
+	bool_t selfview=linphone_gtk_get_ui_config_int("videoselfview",VIDEOSELFVIEW_DEFAULT);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(linphone_gtk_get_widget(
 					linphone_gtk_get_main_window(),
 					audio_only ? "audio_only_item" : "video_item")), TRUE);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(linphone_gtk_get_widget(
+					linphone_gtk_get_main_window(),"selfview_item")),selfview);
 }
 
 static gboolean linphone_gtk_can_manage_accounts(){
@@ -987,12 +1050,18 @@ static void linphone_gtk_configure_main_window(){
 	static const char *home;
 	static const char *start_call_icon;
 	static const char *stop_call_icon;
+	static const char *search_icon;
+	static gboolean update_check_menu;
+	static gboolean buttons_have_borders;
 	GtkWidget *w=linphone_gtk_get_main_window();
 	if (!config_loaded){
 		title=linphone_gtk_get_ui_config("title","Linphone");
 		home=linphone_gtk_get_ui_config("home","http://www.linphone.org");
-		start_call_icon=linphone_gtk_get_ui_config("start_call_icon","green.png");
-		stop_call_icon=linphone_gtk_get_ui_config("stop_call_icon","red.png");
+		start_call_icon=linphone_gtk_get_ui_config("start_call_icon","startcall-green.png");
+		stop_call_icon=linphone_gtk_get_ui_config("stop_call_icon","stopcall-red.png");
+		search_icon=linphone_gtk_get_ui_config("directory_search_icon",NULL);
+		update_check_menu=linphone_gtk_get_ui_config_int("update_check_menu",0);
+		buttons_have_borders=linphone_gtk_get_ui_config_int("buttons_border",1);
 		config_loaded=TRUE;
 	}
 	linphone_gtk_configure_window(w,"main_window");
@@ -1005,12 +1074,20 @@ static void linphone_gtk_configure_main_window(){
 	if (start_call_icon){
 		GdkPixbuf *pbuf=create_pixbuf(start_call_icon);
 		gtk_image_set_from_pixbuf(GTK_IMAGE(linphone_gtk_get_widget(w,"start_call_icon")),pbuf);
+		if (buttons_have_borders)
+			gtk_button_set_relief(GTK_BUTTON(linphone_gtk_get_widget(w,"start_call")),GTK_RELIEF_NORMAL);
 		g_object_unref(G_OBJECT(pbuf));
 	}
 	if (stop_call_icon){
 		GdkPixbuf *pbuf=create_pixbuf(stop_call_icon);
 		gtk_image_set_from_pixbuf(GTK_IMAGE(linphone_gtk_get_widget(w,"terminate_call_icon")),pbuf);
-		gtk_image_set_from_pixbuf(GTK_IMAGE(linphone_gtk_get_widget(w,"in_call_terminate_icon")),pbuf);
+		if (buttons_have_borders)
+			gtk_button_set_relief(GTK_BUTTON(linphone_gtk_get_widget(w,"terminate_call")),GTK_RELIEF_NORMAL);
+		g_object_unref(G_OBJECT(pbuf));
+	}
+	if (search_icon){
+		GdkPixbuf *pbuf=create_pixbuf(search_icon);
+		gtk_image_set_from_pixbuf(GTK_IMAGE(linphone_gtk_get_widget(w,"directory_search_button_icon")),pbuf);
 		g_object_unref(G_OBJECT(pbuf));
 	}
 	if (home){
@@ -1019,8 +1096,25 @@ static void linphone_gtk_configure_main_window(){
 		tmp=g_strdup(home);
 		g_object_set_data(G_OBJECT(menu_item),"home",tmp);
 	}
+	{
+		GdkPixbuf *pbuf=create_pixbuf("contact-orange.png");
+		if (pbuf) {
+			gtk_image_set_from_pixbuf(GTK_IMAGE(linphone_gtk_get_widget(w,"contact_tab_icon")),pbuf);
+			g_object_unref(G_OBJECT(pbuf));
+		}
+	}
+	{
+		GdkPixbuf *pbuf=create_pixbuf("dialer-orange.png");
+		if (pbuf) {
+			gtk_image_set_from_pixbuf(GTK_IMAGE(linphone_gtk_get_widget(w,"keypad_tab_icon")),pbuf);
+			g_object_unref(G_OBJECT(pbuf));
+		}
+	}
 	if (!linphone_gtk_can_manage_accounts())
 		gtk_widget_hide(linphone_gtk_get_widget(w,"run_assistant"));
+	if (update_check_menu){
+		gtk_widget_show(linphone_gtk_get_widget(w,"versioncheck"));
+	}
 }
 
 void linphone_gtk_manage_login(void){
@@ -1036,6 +1130,8 @@ void linphone_gtk_manage_login(void){
 }
 
 static void linphone_gtk_init_main_window(){
+	GtkWidget *main_window;
+
 	linphone_gtk_configure_main_window();
 	linphone_gtk_manage_login();
 	load_uri_history();
@@ -1044,10 +1140,14 @@ static void linphone_gtk_init_main_window(){
 	linphone_gtk_show_friends();
 	linphone_gtk_connect_digits();
 	linphone_gtk_check_menu_items();
-	linphone_gtk_enable_mute_button(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(linphone_gtk_get_main_window(),
+	main_window=linphone_gtk_get_main_window();
+	linphone_gtk_enable_mute_button(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(main_window,
 					"main_mute")),FALSE);
-	linphone_gtk_enable_mute_button(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(linphone_gtk_get_main_window(),
+	linphone_gtk_enable_mute_button(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(main_window,
 					"incall_mute")),FALSE);
+	if (!linphone_gtk_use_in_call_view()) {
+		gtk_widget_show(linphone_gtk_get_widget(main_window, "main_mute"));
+	}
 	if (linphone_core_in_call(linphone_gtk_get_core())) linphone_gtk_call_started(
 		linphone_gtk_get_main_window());/*hide the call button, show terminate button*/
 }
@@ -1114,22 +1214,33 @@ static void linphone_gtk_refer_received(LinphoneCore *lc, const char *refer_to){
 	linphone_gtk_start_call(linphone_gtk_get_main_window());
 }
 
+static void linphone_gtk_check_soundcards(){
+	const char **devices=linphone_core_get_sound_devices(linphone_gtk_get_core());
+	if (devices==NULL || devices[0]==NULL){
+		linphone_gtk_display_something(GTK_MESSAGE_WARNING,
+		    _("No sound cards have been detected on this computer.\n"
+			    "You won't be able to send or receive audio calls."));
+	}
+}
+
 int main(int argc, char *argv[]){
 #ifdef ENABLE_NLS
 	void *p;
 #endif
 	const char *config_file;
+	const char *factory_config_file;
 	const char *lang;
+	GtkSettings *settings;
+	GdkPixbuf *pbuf;
 
 	g_thread_init(NULL);
 	gdk_threads_init();
 	
+	progpath = strdup(argv[0]);
+	
 	config_file=linphone_gtk_get_config_file();
 
 #ifdef WIN32
-	if (workingdir!=NULL)
-		_chdir(workingdir);
-
 	/*workaround for windows: sometimes LANG is defined to an integer value, not understood by gtk */
 	if ((lang=getenv("LANG"))!=NULL){
 		if (atoi(lang)!=0){
@@ -1138,8 +1249,11 @@ int main(int argc, char *argv[]){
 			_putenv(tmp);
 		}
 	}
+#else
+	/*for pulseaudio:*/
+	g_setenv("PULSE_PROP_media.role", "phone", TRUE);
 #endif
-	
+
 	if ((lang=linphone_gtk_get_lang(config_file))!=NULL && lang[0]!='\0'){
 #ifdef WIN32
 		char tmp[128];
@@ -1168,6 +1282,19 @@ int main(int argc, char *argv[]){
 		gdk_threads_leave();
 		return -1;
 	}
+	
+	settings=gtk_settings_get_default();
+	g_object_set(settings, "gtk-menu-images", TRUE, NULL);
+	g_object_set(settings, "gtk-button-images", TRUE, NULL);
+#ifdef WIN32
+	if (workingdir!=NULL)
+		_chdir(workingdir);
+#endif
+	/* Now, look for the factory configuration file, we do it this late
+		 since we want to have had time to change directory and to parse
+		 the options, in case we needed to access the working directory */
+	factory_config_file = linphone_gtk_get_factory_config_file();
+
 	if (linphone_core_wake_up_possible_already_running_instance(
 		config_file, addr_to_call) == 0){
 		g_message("addr_to_call=%s",addr_to_call);
@@ -1180,20 +1307,28 @@ int main(int argc, char *argv[]){
 	add_pixmap_directory("pixmaps");
 	add_pixmap_directory(PACKAGE_DATA_DIR "/pixmaps/linphone");
 
+	
+	g_set_application_name("Linphone");
+	pbuf=create_pixbuf(linphone_gtk_get_ui_config("icon",LINPHONE_ICON));
+	if (pbuf!=NULL) gtk_window_set_default_icon(pbuf);
+	
 	the_ui=linphone_gtk_create_window("main");
 	
 	linphone_gtk_create_log_window();
 	linphone_core_enable_logs_with_cb(linphone_gtk_log_handler);
 
-	linphone_gtk_init_liblinphone(config_file);
+	linphone_gtk_init_liblinphone(config_file, factory_config_file);
 	/* do not lower timeouts under 30 ms because it exhibits a bug on gtk+/win32, with cpu running 20% all the time...*/
 	gtk_timeout_add(30,(GtkFunction)linphone_gtk_iterate,(gpointer)linphone_gtk_get_core());
 	gtk_timeout_add(30,(GtkFunction)linphone_gtk_check_logs,(gpointer)NULL);
 	linphone_gtk_init_main_window();
 	linphone_gtk_init_status_icon();
-	if (!iconified)
+	if (!iconified){
 		linphone_gtk_show_main_window();
-	linphone_gtk_check_for_new_version();
+		linphone_gtk_check_soundcards();
+	}
+	if (linphone_gtk_get_ui_config_int("update_check_menu",0)==0)
+		linphone_gtk_check_for_new_version();
 
 	gtk_main();
 	gdk_threads_leave();
@@ -1201,5 +1336,6 @@ int main(int argc, char *argv[]){
 	linphone_core_destroy(the_core);
 	/*workaround a bug on win32 that makes status icon still present in the systray even after program exit.*/
 	gtk_status_icon_set_visible(icon,FALSE);
+	free(progpath);
 	return 0;
 }
