@@ -802,8 +802,10 @@ void linphone_gtk_show_parameters(void){
 	/* SIP CONFIG */
 	contact=linphone_core_get_primary_contact_parsed(lc);
 	if (contact){
-		if (linphone_address_get_display_name(contact)) 
-			gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(pb,"displayname")),linphone_address_get_display_name(contact));
+		if (linphone_address_get_display_name(contact)) {
+			const char *dn=linphone_address_get_display_name(contact);
+			gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(pb,"displayname")),dn);
+		}
 		if (linphone_address_get_username(contact))
 			gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(pb,"username")),linphone_address_get_username(contact));
 	}
