@@ -39,7 +39,7 @@ time_t ms_time (time_t *t);
 typedef enum EchoLimiterType{
 	ELInactive,
 	ELControlMic,
-	ELControlSpeaker
+	ELControlFull
 } EchoLimiterType;
 
 struct _AudioStream
@@ -129,6 +129,9 @@ void audio_stream_set_echo_canceller_params(AudioStream *st, int tail_len_ms, in
 
 void audio_stream_set_mic_gain(AudioStream *stream, float gain);
 
+/* enable/disable rtp stream */ 
+void audio_stream_mute_rtp(AudioStream *stream, bool_t val);
+
 /*enable noise gate, must be done before start()*/
 void audio_stream_enable_noise_gate(AudioStream *stream, bool_t val);
 
@@ -158,6 +161,7 @@ void audio_stream_get_local_rtp_stats(AudioStream *stream, rtp_stats_t *stats);
   Video Support
  *****************/
 
+typedef void (*VideoStreamRenderCallback)(void *user_pointer, const MSPicture *local_view, const MSPicture *remote_view);
 
 struct _VideoStream
 {
@@ -175,6 +179,9 @@ struct _VideoStream
 	OrtpEvQueue *evq;
 	MSVideoSize sent_vsize;
 	int corner; /*for selfview*/
+	VideoStreamRenderCallback rendercb;
+	void *render_pointer;
+	char *display_name;
 	bool_t adapt_bitrate;
 };
 
@@ -182,6 +189,8 @@ typedef struct _VideoStream VideoStream;
 
 VideoStream *video_stream_new(int locport, bool_t use_ipv6);
 void video_stream_enable_adaptive_bitrate_control(VideoStream *s, bool_t yesno);
+void video_stream_set_render_callback(VideoStream *s, VideoStreamRenderCallback cb, void *user_pointer);
+void video_stream_set_display_filter_name(VideoStream *s, const char *fname);
 int video_stream_start(VideoStream * stream, RtpProfile *profile, const char *remip, int remport, int rem_rtcp_port,
 		int payload, int jitt_comp, MSWebCam *device);
 void video_stream_set_relay_session_id(VideoStream *stream, const char *relay_session_id);
