@@ -167,23 +167,15 @@ void ms_exit(void);
 
 struct _MSSndCardDesc;
 
-/**
- * This can be called when new sound cards have been added.
- *
- * Warning: It must NOT be called if one filter is currently created.
- * You have to unlink, destroy everything before doing this.
- *
- * @param snd_desc   A MSSndCardDesc description.
- */
-void ms_reload_snd_card(struct _MSSndCardDesc *snd_desc);
-
 void ms_sleep(int seconds);
 
+void ms_usleep(uint64_t usec);
+
 /**
  * The max payload size allowed.
  * Filters that generate data that can be sent through RTP should make packets
  * whose size is below ms_get_payload_max_size().
- * The default value is 1440 computed as the standart internet MTU minus IPv6 header,
+ * The default value is 1440 computed as the standard internet MTU minus IPv6 header,
  * UDP header and RTP header. As IPV4 header is smaller than IPv6 header, this 
  * value works for both.
  * 
