@@ -23,33 +23,66 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "msfilter.h"
 
 /* some global constants for video MSFilter(s) */
-#define MS_VIDEO_SIZE_CIF_W 352
-#define MS_VIDEO_SIZE_CIF_H 288
+#define MS_VIDEO_SIZE_SQCIF_W 128
+#define MS_VIDEO_SIZE_SQCIF_H 96
+#define MS_VIDEO_SIZE_WQCIF_W 256
+#define MS_VIDEO_SIZE_WQCIF_H 144
 #define MS_VIDEO_SIZE_QCIF_W 176
 #define MS_VIDEO_SIZE_QCIF_H 144
+#define MS_VIDEO_SIZE_CIF_W 352
+#define MS_VIDEO_SIZE_CIF_H 288
+#define MS_VIDEO_SIZE_ICIF_W 352
+#define MS_VIDEO_SIZE_ICIF_H 576
 #define MS_VIDEO_SIZE_4CIF_W 704
 #define MS_VIDEO_SIZE_4CIF_H 576
+#define MS_VIDEO_SIZE_W4CIF_W 1024
+#define MS_VIDEO_SIZE_W4CIF_H 576
 
 #define MS_VIDEO_SIZE_QQVGA_W 160
 #define MS_VIDEO_SIZE_QQVGA_H 120
-
 #define MS_VIDEO_SIZE_QVGA_W 320
 #define MS_VIDEO_SIZE_QVGA_H 240
-
 #define MS_VIDEO_SIZE_VGA_W 640
 #define MS_VIDEO_SIZE_VGA_H 480
-
-#define MS_VIDEO_SIZE_720P_W 1280
-#define MS_VIDEO_SIZE_720P_H 720
+#define MS_VIDEO_SIZE_SVGA_W 800
+#define MS_VIDEO_SIZE_SVGA_H 600
 
 #define MS_VIDEO_SIZE_NS1_W 324
 #define MS_VIDEO_SIZE_NS1_H 248
 
-#define MS_VIDEO_SIZE_SVGA_W 800
-#define MS_VIDEO_SIZE_SVGA_H 600
+#define MS_VIDEO_SIZE_QSIF_W 176
+#define MS_VIDEO_SIZE_QSIF_H 120
+#define MS_VIDEO_SIZE_SIF_W 352
+#define MS_VIDEO_SIZE_SIF_H 240
+#define MS_VIDEO_SIZE_ISIF_W 352
+#define MS_VIDEO_SIZE_ISIF_H 480
+#define MS_VIDEO_SIZE_4SIF_W 704
+#define MS_VIDEO_SIZE_4SIF_H 480
+
+#define MS_VIDEO_SIZE_288P_W 512
+#define MS_VIDEO_SIZE_288P_H 288
+#define MS_VIDEO_SIZE_432P_W 768
+#define MS_VIDEO_SIZE_432P_H 432
+#define MS_VIDEO_SIZE_448P_W 768
+#define MS_VIDEO_SIZE_448P_H 448
+#define MS_VIDEO_SIZE_480P_W 848
+#define MS_VIDEO_SIZE_480P_H 480
+#define MS_VIDEO_SIZE_576P_W 1024
+#define MS_VIDEO_SIZE_576P_H 576
+#define MS_VIDEO_SIZE_720P_W 1280
+#define MS_VIDEO_SIZE_720P_H 720
+#define MS_VIDEO_SIZE_1080P_W 1920
+#define MS_VIDEO_SIZE_1080P_H 1080
+
+#define MS_VIDEO_SIZE_SDTV_W 768
+#define MS_VIDEO_SIZE_SDTV_H 576
+#define MS_VIDEO_SIZE_HDTVP_W 1920
+#define MS_VIDEO_SIZE_HDTVP_H 1200
 
 #define MS_VIDEO_SIZE_XGA_W 1024
 #define MS_VIDEO_SIZE_XGA_H 768
+#define MS_VIDEO_SIZE_WXGA_W 1080
+#define MS_VIDEO_SIZE_WXGA_H 768
 
 #define MS_VIDEO_SIZE_MAX_W MS_VIDEO_SIZE_1024_W
 #define MS_VIDEO_SIZE_MAX_H MS_VIDEO_SIZE_1024_H
@@ -98,6 +131,7 @@ typedef enum{
 	MS_MJPEG,
 	MS_UYVY,
 	MS_YUY2,   /* -> same as MS_YUYV */
+	MS_RGBA32,
 	MS_PIX_FMT_UNKNOWN
 }MSPixFmt;
 
@@ -120,9 +154,10 @@ void ms_ffmpeg_check_init(void);
 int yuv_buf_init_from_mblk(MSPicture *buf, mblk_t *m);
 void yuv_buf_init_from_mblk_with_size(MSPicture *buf, mblk_t *m, int w, int h);
 mblk_t * yuv_buf_alloc(MSPicture *buf, int w, int h);
-void yuv_buf_copy(uint8_t *src_planes[], const int src_strides[], 
+void ms_yuv_buf_copy(uint8_t *src_planes[], const int src_strides[], 
 		uint8_t *dst_planes[], const int dst_strides[3], MSVideoSize roi);
-void yuv_buf_mirror(YuvBuf *buf);
+void ms_yuv_buf_mirror(YuvBuf *buf);
+void rgb24_mirror(uint8_t *buf, int w, int h, int linesize);
 void rgb24_revert(uint8_t *buf, int w, int h, int linesize);
 void rgb24_copy_revert(uint8_t *dstbuf, int dstlsz,
 				const uint8_t *srcbuf, int srclsz, MSVideoSize roi);
@@ -145,6 +180,38 @@ static inline bool_t ms_video_size_equal(MSVideoSize vs1, MSVideoSize vs2){
 
 MSVideoSize ms_video_size_get_just_lower_than(MSVideoSize vs);
 
+struct ms_SwsContext;
+struct _SwsFilter;
+
+struct ms_SwsContext *ms_sws_getContext(int srcW, int srcH, int srcFormat,
+                                  int dstW, int dstH, int dstFormat,
+                                  int flags, struct _SwsFilter *srcFilter,
+                                  struct _SwsFilter *dstFilter, double *param);
+void ms_sws_freeContext(struct ms_SwsContext *swsContext);
+int ms_sws_scale(struct ms_SwsContext *context, uint8_t* srcSlice[], int srcStride[],
+              int srcSliceY, int srcSliceH, uint8_t* dst[], int dstStride[]);
+
+typedef struct ms_SwsContext *(*sws_getContextFunc)(int srcW, int srcH, int srcFormat,
+                                  int dstW, int dstH, int dstFormat,
+                                  int flags, struct _SwsFilter *srcFilter,
+                                  struct _SwsFilter *dstFilter, double *param);
+typedef void (*sws_freeContextFunc)(struct ms_SwsContext *swsContext);
+typedef int (*sws_scaleFunc)(struct ms_SwsContext *context, uint8_t* srcSlice[], int srcStride[],
+              int srcSliceY, int srcSliceH, uint8_t* dst[], int dstStride[]);
+typedef void (*yuv_buf_mirrorFunc)(MSPicture *buf);
+typedef void (*yuv_buf_copyFunc)(uint8_t *src_planes[], const int src_strides[], 
+		uint8_t *dst_planes[], const int dst_strides[3], MSVideoSize roi);
+
+struct ms_swscaleDesc {
+	sws_getContextFunc sws_getContext;
+	sws_freeContextFunc sws_freeContext;
+	sws_scaleFunc sws_scale;
+	yuv_buf_mirrorFunc yuv_buf_mirror;
+	yuv_buf_copyFunc yuv_buf_copy;
+};
+
+void ms_video_set_video_func(struct ms_swscaleDesc *_ms_swscale_desc);
+
 #ifdef __cplusplus
 }
 #endif
