@@ -51,6 +51,25 @@ struct _AlsaData{
 typedef struct _AlsaData AlsaData;
 
 
+static void alsa_resume(snd_pcm_t *handle){
+	int err;
+	snd_pcm_status_t *status=NULL;
+
+	snd_pcm_status_alloca(&status);
+	
+	if ((err=snd_pcm_status(handle,status))!=0){
+		ms_warning("snd_pcm_status() failed: %s",snd_strerror(err));
+		return;
+	}
+
+	if (snd_pcm_status_get_state(status)==SND_PCM_STATE_SUSPENDED){
+		ms_warning("Maybe suspended, trying resume");
+		if ((err=snd_pcm_resume(handle))!=0){
+			if (err!=EWOULDBLOCK) ms_warning("snd_pcm_resume() failed: %s",snd_strerror(err));
+		}
+	}
+}
+
 static int alsa_set_params(snd_pcm_t *pcm_handle, int rw, int bits, int stereo, int rate)
 {
 	snd_pcm_hw_params_t *hwparams=NULL;
@@ -208,7 +227,7 @@ static snd_pcm_t * alsa_open_r(const char *pcmdev,int bits,int stereo,int rate)
 		return NULL;
 	}
 #endif
-
+	alsa_resume(pcm_handle);
 	{
 	struct timeval tv1;
 	struct timeval tv2;
@@ -245,12 +264,14 @@ static snd_pcm_t * alsa_open_r(const char *pcmdev,int bits,int stereo,int rate)
 static snd_pcm_t * alsa_open_w(const char *pcmdev,int bits,int stereo,int rate)
 {
 	snd_pcm_t *pcm_handle;
+
+	ms_message("alsa_open_r: opening %s at %iHz, bits=%i, stereo=%i",pcmdev,rate,bits,stereo);
 	
 	if (snd_pcm_open(&pcm_handle, pcmdev,SND_PCM_STREAM_PLAYBACK,SND_PCM_NONBLOCK) < 0) {
 		ms_warning("alsa_open_w: Error opening PCM device %s",pcmdev );
 		return NULL;
 	}
-	
+	alsa_resume(pcm_handle);
 	{
 	struct timeval tv1;
 	struct timeval tv2;
@@ -310,13 +331,6 @@ static int alsa_can_read(snd_pcm_t *dev)
 	return avail;
 }
 
-static void alsa_resume(snd_pcm_t *handle){
-	int err;
-	ms_warning("Maybe suspended, trying resume");
-	if ((err=snd_pcm_resume(handle))!=0){
-		if (err!=EWOULDBLOCK) ms_warning("snd_pcm_resume() failed: %s",snd_strerror(err));
-	}
-}
 
 static int alsa_read(snd_pcm_t *handle,unsigned char *buf,int nsamples)
 {
@@ -870,7 +884,7 @@ void alsa_read_process(MSFilter *obj){
 	if (ad->handle==NULL) return;
 	while (alsa_can_read(ad->handle)>=samples){
 	  
-		int size=samples*2;
+		int size=samples*2*ad->nchannels;
 		om=allocb(size,0);
 		if ((err=alsa_read(ad->handle,om->b_wptr,samples))<=0) {
 			ms_warning("Fail to read samples");
@@ -890,13 +904,14 @@ void alsa_read_process(MSFilter *obj){
 	AlsaReadData *ad=(AlsaReadData*)obj->data;
 	mblk_t *om=NULL;
 	int samples=(160*ad->rate)/8000;
-
+	int size=samples*2*ad->nchannels;
+	
 	ms_mutex_lock(&ad->mutex);
-	while (ms_bufferizer_get_avail(ad->bufferizer)>=samples*2){
+	while (ms_bufferizer_get_avail(ad->bufferizer)>=size){
 	  
-	  om=allocb(samples*2,0);
-	  ms_bufferizer_read(ad->bufferizer,om->b_wptr,samples*2);	  
-	  om->b_wptr+=samples*2;
+	  om=allocb(size,0);
+	  ms_bufferizer_read(ad->bufferizer,om->b_wptr,size);	  
+	  om->b_wptr+=size;
 	  /*ms_message("alsa_read_process: Outputing %i bytes",size);*/
 	  ms_queue_put(obj->outputs[0],om);
 	}
