@@ -31,7 +31,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msfilerec.h"
 #include "mediastreamer2/msvolume.h"
 #include "mediastreamer2/msequalizer.h"
-#include "mediastreamer2/msspeexec.h"
 
 #ifdef INET6
 	#include <sys/types.h>
@@ -250,20 +249,23 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		stream->ec=ms_filter_new(MS_SPEEX_EC_ID);
 		ms_filter_call_method(stream->ec,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
 		if (stream->ec_tail_len!=0)
-			ms_filter_call_method(stream->ec,MS_SPEEX_EC_SET_TAIL_LENGTH,&stream->ec_tail_len);
+			ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_TAIL_LENGTH,&stream->ec_tail_len);
 		if (stream->ec_delay!=0)
-			ms_filter_call_method(stream->ec,MS_SPEEX_EC_SET_DELAY,&stream->ec_delay);
+			ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_DELAY,&stream->ec_delay);
 		if (stream->ec_framesize!=0)
-			ms_filter_call_method(stream->ec,MS_SPEEX_EC_SET_FRAME_SIZE,&stream->ec_framesize);
+			ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_FRAMESIZE,&stream->ec_framesize);
 	}
 
 	if (stream->el_type!=ELInactive || stream->use_gc || stream->use_ng){
 		stream->volsend=ms_filter_new(MS_VOLUME_ID);
 		stream->volrecv=ms_filter_new(MS_VOLUME_ID);
 		if (stream->el_type!=ELInactive){
-			if (stream->el_type==ELControlSpeaker)
-				ms_filter_call_method(stream->volrecv,MS_VOLUME_SET_PEER,stream->volsend);
-			else ms_filter_call_method(stream->volsend,MS_VOLUME_SET_PEER,stream->volrecv);
+			if (stream->el_type==ELControlFull) {
+				/* also reduce speaker gain when no signal - same parameters as std. noise gate */
+				int tmp=1;
+				ms_filter_call_method(stream->volrecv,MS_VOLUME_ENABLE_NOISE_GATE,&tmp);
+			}
+			ms_filter_call_method(stream->volsend,MS_VOLUME_SET_PEER,stream->volrecv);
 		}
 		if (stream->use_ng){
 			int tmp=1;
@@ -598,3 +600,14 @@ void audio_stream_get_local_rtp_stats(AudioStream *stream, rtp_stats_t *lstats){
 		memcpy(lstats,stats,sizeof(*stats));
 	}else memset(lstats,0,sizeof(rtp_stats_t));
 }
+
+
+void audio_stream_mute_rtp(AudioStream *stream, bool_t val) 
+{
+  if (stream->rtpsend){
+    if (val)
+      ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_MUTE_MIC,&val);
+    else
+      ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_UNMUTE_MIC,&val);
+  }
+}
