@@ -146,6 +146,9 @@ static LRESULT CALLBACK window_proc(
 	DDDisplay *wd=(DDDisplay*)GetWindowLongPtr(hwnd,GWLP_USERDATA);
 	switch(uMsg){
 		case WM_DESTROY:
+			if (wd){
+				wd->window=NULL;
+			}
 		break;
 		case WM_SIZE:
 			if (wParam==SIZE_RESTORED){
@@ -464,7 +467,13 @@ static void dd_display_process(MSFilter *f){
 	int corner=obj->sv_corner;
 	float scalefactor=obj->sv_scalefactor;
 
-	GetClientRect(obj->window,&rect);
+	if (obj->window==NULL){
+		goto end;
+	}
+
+	if (GetClientRect(obj->window,&rect)==0
+	    || rect.right<=32 || rect.bottom<=32) goto end;
+
 	wsize.width=rect.right;
 	wsize.height=rect.bottom;
 	obj->wsize=wsize;
@@ -558,11 +567,14 @@ static void dd_display_process(MSFilter *f){
 		/*else using direct blitting to screen*/
 
 		ReleaseDC(NULL,hdc);
-		if (main_im!=NULL)
-			ms_queue_flush(f->inputs[0]);
-		if (local_im!=NULL)
-			ms_queue_flush(f->inputs[1]);
 	}
+	
+	end:
+		
+	if (f->inputs[0]!=NULL)
+		ms_queue_flush(f->inputs[0]);
+	if (f->inputs[1]!=NULL)
+		ms_queue_flush(f->inputs[1]);
 }
 
 static int get_native_window_id(MSFilter *f, void *data){
