@@ -48,9 +48,13 @@ extern void __register_ffmpeg_encoders_if_possible(void);
 #include <dlfcn.h>
 #endif
 
-#ifdef __APPLE__
+#if defined(__APPLE__) && !defined(__GNUC__)
 #import <Cocoa/Cocoa.h>
-#include <Foundation/Foundation.h>
+#include <Foundation/Foundation.h> 
+#endif
+
+#ifdef ANDROID
+#include <android/log.h>
 #endif
 
 MSList *ms_list_new(void *data){
@@ -257,87 +261,79 @@ MSList *ms_list_copy(const MSList *list){
 typedef void (*init_func_t)(void);
 
 int ms_load_plugins(const char *dir){
-    int num=0;
+	int num=0;
 #if defined(WIN32) && !defined(_WIN32_WCE)
-    WIN32_FIND_DATA FileData; 
-    HANDLE hSearch; 
-    char szDirPath[1024]; 
-    char szPluginFile[1024]; 
-    BOOL fFinished = FALSE;
-    
+	WIN32_FIND_DATA FileData; 
+	HANDLE hSearch; 
+	char szDirPath[1024]; 
+	char szPluginFile[1024]; 
+	BOOL fFinished = FALSE;
+	const char *tmp=getenv("DEBUG");
+	BOOL debug=(tmp!=NULL && atoi(tmp)==1);
 	snprintf(szDirPath, sizeof(szDirPath), "%s", dir);
-    // Create a new directory.    
-#if 0
-    if (!CreateDirectory(szDirPath, NULL))
-    {
-        ms_message("plugins directory already exist (%s).", szDirPath);
-    }
-#endif
-    
-    // Start searching for .TXT files in the current directory.
-    
+
+	// Start searching for .dll files in the current directory.
+
 	snprintf(szDirPath, sizeof(szDirPath), "%s\\*.dll", dir);
-    hSearch = FindFirstFile(szDirPath, &FileData);
-    if (hSearch == INVALID_HANDLE_VALUE)
-    {
-        ms_message("no plugin (*.dll) found in %s.", szDirPath);
+	hSearch = FindFirstFile(szDirPath, &FileData);
+	if (hSearch == INVALID_HANDLE_VALUE)
+	{
+		ms_message("no plugin (*.dll) found in %s.", szDirPath);
 		return 0;
-    }
+	}
 	snprintf(szDirPath, sizeof(szDirPath), "%s", dir);
 
-    while (!fFinished) 
-    {
-        /* load library */
-        HINSTANCE os_handle;
-        UINT em;
-        em = SetErrorMode (SEM_FAILCRITICALERRORS);
-
-        snprintf(szPluginFile, sizeof(szPluginFile), "%s\\%s", szDirPath, FileData.cFileName);
-        os_handle = LoadLibraryEx (szPluginFile, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
-        if (os_handle==NULL)
-        {
-			ms_warning("Fail to load plugin %s: error %i",szPluginFile,GetLastError());
-            os_handle = LoadLibraryEx (szPluginFile, NULL, 0);
-        }
-        SetErrorMode (em);
-        if (os_handle==NULL)
-            ms_warning("Fail to load plugin %s", szPluginFile); 
-        else
-        {
-            init_func_t initroutine;
-            char szPluginName[256]; 
-            char szMethodName[256]; 
-            snprintf(szPluginName, 256, "%s", FileData.cFileName);
-            szPluginName[strlen(szPluginName)-4]='\0';
-            snprintf(szMethodName, 256, "%s_init", szPluginName);
-            initroutine = (init_func_t) GetProcAddress (os_handle, szMethodName);
-			if (initroutine!=NULL){
-				initroutine();
-				ms_message("Plugin loaded (%s)", szPluginFile);
-				num++;
-			}else{
-				ms_warning("Could not locate init routine of plugin %s", szPluginFile);
-			}
-        }
-
-
-        if (!FindNextFile(hSearch, &FileData)) 
-        {
-            if (GetLastError() == ERROR_NO_MORE_FILES) 
-            { 
-                fFinished = TRUE; 
-            } 
-            else 
-            { 
-                ms_error("couldn't find next plugin dll."); 
-                fFinished = TRUE; 
-            } 
-        }
-    } 
-     
-    // Close the search handle. 
-     
-    FindClose(hSearch);
+	while (!fFinished) 
+	{
+		/* load library */
+		HINSTANCE os_handle;
+		UINT em;
+		if (!debug) em = SetErrorMode (SEM_FAILCRITICALERRORS);
+		
+		snprintf(szPluginFile, sizeof(szPluginFile), "%s\\%s", szDirPath, FileData.cFileName);
+		os_handle = LoadLibraryEx (szPluginFile, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
+		if (os_handle==NULL)
+		{
+			ms_message("Fail to load plugin %s with altered search path: error %i",szPluginFile,GetLastError());
+			os_handle = LoadLibraryEx (szPluginFile, NULL, 0);
+		}
+		if (!debug) SetErrorMode (em);
+		if (os_handle==NULL)
+			ms_error("Fail to load plugin %s", szPluginFile); 
+		else{
+			init_func_t initroutine;
+			char szPluginName[256]; 
+			char szMethodName[256];
+			char *minus;
+			snprintf(szPluginName, 256, "%s", FileData.cFileName);
+			/*on mingw, dll names might be libsomething-3.dll. We must skip the -X.dll stuff*/
+			minus=strchr(szPluginName,'-');
+			if (minus) *minus='\0';
+			else szPluginName[strlen(szPluginName)-4]='\0'; /*remove .dll*/
+			snprintf(szMethodName, 256, "%s_init", szPluginName);
+			initroutine = (init_func_t) GetProcAddress (os_handle, szMethodName);
+				if (initroutine!=NULL){
+					initroutine();
+					ms_message("Plugin loaded (%s)", szPluginFile);
+					num++;
+				}else{
+					ms_warning("Could not locate init routine of plugin %s. Should be %s",
+					szPluginFile, szMethodName);
+				}
+		}
+		if (!FindNextFile(hSearch, &FileData)) {
+			if (GetLastError() == ERROR_NO_MORE_FILES){ 
+				fFinished = TRUE; 
+			} 
+			else 
+			{ 
+				ms_error("couldn't find next plugin dll."); 
+				fFinished = TRUE; 
+			} 
+		}
+	} 
+	/* Close the search handle. */
+	FindClose(hSearch);
 
 #elif HAVE_DLOPEN
 	DIR *ds;
@@ -363,22 +359,21 @@ int ms_load_plugins(const char *dir){
 				void *initroutine=NULL;
 				strcpy(initroutine_name,de->d_name);
 				p=strstr(initroutine_name,PLUGINS_EXT);
-				if (p!=NULL)
-				  {
-				    strcpy(p,"_init");
-				    initroutine=dlsym(handle,initroutine_name);
-				  }
+				if (p!=NULL){
+					strcpy(p,"_init");
+					initroutine=dlsym(handle,initroutine_name);
+				}
 
 #ifdef __APPLE__
 				if (initroutine==NULL){
-				  /* on macosx: library name are libxxxx.1.2.3.dylib */
-				  /* -> MUST remove the .1.2.3 */
-				  p=strstr(initroutine_name,".");
-				  if (p!=NULL)
-				    {
-				      strcpy(p,"_init");
-				      initroutine=dlsym(handle,initroutine_name);
-				    }
+					/* on macosx: library name are libxxxx.1.2.3.dylib */
+					/* -> MUST remove the .1.2.3 */
+					p=strstr(initroutine_name,".");
+					if (p!=NULL)
+					{
+						strcpy(p,"_init");
+						initroutine=dlsym(handle,initroutine_name);
+					}
 				}
 #endif
 
@@ -436,6 +431,10 @@ extern MSSndCardDesc pasnd_card_desc;
 extern MSSndCardDesc aq_card_desc;
 #endif
 
+#ifdef __PULSEAUDIO_ENABLED__
+extern MSSndCardDesc pulse_card_desc;
+#endif
+
 static MSSndCardDesc * ms_snd_card_descs[]={
 #ifdef __ALSA_ENABLED__
 	&alsa_card_desc,
@@ -460,6 +459,9 @@ static MSSndCardDesc * ms_snd_card_descs[]={
 #endif
 #ifdef __MAC_AQ_ENABLED__
 	&aq_card_desc,
+#endif
+#ifdef __PULSEAUDIO_ENABLED__
+	&pulse_card_desc,
 #endif
 	NULL
 };
@@ -471,7 +473,7 @@ extern MSWebCamDesc v4l_desc;
 #endif
 
 #ifdef HAVE_LINUX_VIDEODEV2_H
-extern MSWebCamDesc v4l2_desc;
+extern MSWebCamDesc v4l2_card_desc;
 #endif
 
 #ifdef WIN32
@@ -481,12 +483,22 @@ extern MSWebCamDesc ms_vfw_cam_desc;
 extern MSWebCamDesc ms_directx_cam_desc;
 #endif
 
+#ifdef __MINGW32__
+extern MSWebCamDesc ms_dshow_cam_desc;
+#endif
+
+#ifdef __APPLE__
+extern MSWebCamDesc ms_v4m_cam_desc;
+#endif
+
+#if !defined(NO_FFMPEG)
 extern MSWebCamDesc static_image_desc;
 extern MSWebCamDesc mire_desc;
+#endif
 
 static MSWebCamDesc * ms_web_cam_descs[]={
 #ifdef HAVE_LINUX_VIDEODEV2_H
-	&v4l2_desc,
+	&v4l2_card_desc,
 #endif
 #ifdef __linux
 	&v4l_desc,
@@ -497,24 +509,50 @@ static MSWebCamDesc * ms_web_cam_descs[]={
 #if defined(WIN32) && defined(HAVE_VFW)
 	&ms_vfw_cam_desc,
 #endif
+#ifdef __MINGW32__
+	&ms_dshow_cam_desc,
+#endif
+#ifdef __APPLE__
+	&ms_v4m_cam_desc,
+#endif
+
+#if !defined(NO_FFMPEG)
 	&mire_desc,
 	&static_image_desc,
+#endif
 	NULL
 };
 
 #endif
 
+#ifdef ANDROID
+#define LOG_DOMAIN "mediastreamer"
+static void ms_android_log_handler(OrtpLogLevel lev, const char *fmt, va_list args){
+	int prio;
+	switch(lev){
+	case ORTP_DEBUG:	prio = ANDROID_LOG_DEBUG;	break;
+	case ORTP_MESSAGE:	prio = ANDROID_LOG_INFO;	break;
+	case ORTP_WARNING:	prio = ANDROID_LOG_WARN;	break;
+	case ORTP_ERROR:	prio = ANDROID_LOG_ERROR;	break;
+	case ORTP_FATAL:	prio = ANDROID_LOG_FATAL;	break;
+	default:		prio = ANDROID_LOG_DEFAULT;	break;
+	}
+	__android_log_vprint(prio, LOG_DOMAIN, fmt, args);
+}
+#endif
+
 void ms_init(){
 	int i;
 	MSSndCardManager *cm;
-#ifdef __APPLE__
-	NSApplicationLoad();
-#endif
 
 #if !defined(_WIN32_WCE)
 	if (getenv("MEDIASTREAMER_DEBUG")!=NULL){
 		ortp_set_log_level_mask(ORTP_DEBUG|ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
 	}
+#endif
+#ifdef ANDROID
+	ortp_set_log_level_mask(ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
+	ortp_set_log_handler(ms_android_log_handler);
 #endif
 	ms_message("Registering all filters...");
 	/* register builtin MSFilter's */
@@ -536,7 +574,9 @@ void ms_init(){
 			ms_web_cam_manager_register_desc(wm,ms_web_cam_descs[i]);
 		}
 	}
+#if !defined(NO_FFMPEG)
 	__register_ffmpeg_encoders_if_possible();
+#endif
 #endif
 	ms_message("Loading plugins");
 	ms_load_plugins(PACKAGE_PLUGINS_DIR);
@@ -566,6 +606,21 @@ void ms_sleep(int seconds){
 #endif
 }
 
+void ms_usleep(uint64_t usec){
+#ifdef WIN32
+	Sleep((DWORD)(usec/1000));
+#else
+	struct timespec ts,rem;
+	int err;
+	ts.tv_sec=usec/1000000LL;
+	ts.tv_nsec=(usec%1000000LL)*1000;
+	do {
+		err=nanosleep(&ts,&rem);
+		ts=rem;
+	}while(err==-1 && errno==EINTR);
+#endif
+}
+
 #define DEFAULT_MAX_PAYLOAD_SIZE 1440
 
 static int max_payload_size=DEFAULT_MAX_PAYLOAD_SIZE;
@@ -578,3 +633,7 @@ void ms_set_payload_max_size(int size){
 	if (size<=0) size=DEFAULT_MAX_PAYLOAD_SIZE;
 	max_payload_size=size;
 }
+
+
+
+
