@@ -123,6 +123,7 @@ MSFilterId ms_filter_get_id(MSFilter *f){
 
 int ms_filter_link(MSFilter *f1, int pin1, MSFilter *f2, int pin2){
 	MSQueue *q;
+	ms_message("ms_filter_link: %s:%p,%i-->%s:%p,%i",f1->desc->name,f1,pin1,f2->desc->name,f2,pin2);
 	ms_return_val_if_fail(pin1<f1->desc->noutputs, -1);
 	ms_return_val_if_fail(pin2<f2->desc->ninputs, -1);
 	ms_return_val_if_fail(f1->outputs[pin1]==NULL,-1);
@@ -130,7 +131,6 @@ int ms_filter_link(MSFilter *f1, int pin1, MSFilter *f2, int pin2){
 	q=ms_queue_new(f1,pin1,f2,pin2);
 	f1->outputs[pin1]=q;
 	f2->inputs[pin2]=q;
-	ms_message("ms_filter_link: %s:%p,%i-->%s:%p,%i",f1->desc->name,f1,pin1,f2->desc->name,f2,pin2);
 	return 0;
 }
 
@@ -138,6 +138,7 @@ int ms_filter_unlink(MSFilter *f1, int pin1, MSFilter *f2, int pin2){
 	MSQueue *q;
 	ms_return_val_if_fail(f1, -1);
 	ms_return_val_if_fail(f2, -1);
+	ms_message("ms_filter_unlink: %s:%p,%i-->%s:%p,%i",f1->desc->name,f1,pin1,f2->desc->name,f2,pin2);
 	ms_return_val_if_fail(pin1<f1->desc->noutputs, -1);
 	ms_return_val_if_fail(pin2<f2->desc->ninputs, -1);
 	ms_return_val_if_fail(f1->outputs[pin1]!=NULL,-1);
@@ -146,23 +147,27 @@ int ms_filter_unlink(MSFilter *f1, int pin1, MSFilter *f2, int pin2){
 	q=f1->outputs[pin1];
 	f1->outputs[pin1]=f2->inputs[pin2]=0;
 	ms_queue_destroy(q);
-	ms_message("ms_filter_unlink: %s:%p,%i-->%s:%p,%i",f1->desc->name,f1,pin1,f2->desc->name,f2,pin2);
 	return 0;
 }
 
 #define MS_FILTER_METHOD_GET_FID(id)	(((id)>>16) & 0xFFFF)
+#define MS_FILTER_METHOD_GET_INDEX(id) ( ((id)>>8) & 0XFF) 
+
+static inline bool_t is_interface_method(unsigned int magic){
+	return magic==MS_FILTER_BASE_ID || magic>MSFilterInterfaceBegin;
+}
 
 int ms_filter_call_method(MSFilter *f, unsigned int id, void *arg){
 	MSFilterMethod *methods=f->desc->methods;
 	int i;
 	unsigned int magic=MS_FILTER_METHOD_GET_FID(id);
-	if (magic!=MS_FILTER_BASE_ID && magic!=f->desc->id) {
+	if (!is_interface_method(magic) && magic!=f->desc->id) {
 		ms_fatal("Method type checking failed when calling %u on filter %s",id,f->desc->name);
 		return -1;
 	}
 	for(i=0;methods!=NULL && methods[i].method!=NULL; i++){
 		unsigned int mm=MS_FILTER_METHOD_GET_FID(methods[i].id);
-		if (mm!=f->desc->id && mm!=MS_FILTER_BASE_ID) {
+		if (mm!=f->desc->id && !is_interface_method(mm)) {
 			ms_fatal("Bad method definition on filter %s. fid=%u , mm=%u",f->desc->name,f->desc->id,mm);
 			return -1;
 		}
@@ -170,7 +175,8 @@ int ms_filter_call_method(MSFilter *f, unsigned int id, void *arg){
 			return methods[i].method(f,arg);
 		}
 	}
-	if (magic!=MS_FILTER_BASE_ID) ms_error("no such method on filter %s",f->desc->name);
+	if (magic!=MS_FILTER_BASE_ID) ms_error("no such method on filter %s, fid=%i method index=%i",f->desc->name,magic,
+	                           MS_FILTER_METHOD_GET_INDEX(id) );
 	return -1;
 }
 
@@ -183,6 +189,10 @@ void ms_filter_set_notify_callback(MSFilter *f, MSFilterNotifyFunc fn, void *ud)
 	f->notify_ud=ud;
 }
 
+void ms_filter_enable_synchronous_notifcations(MSFilter *f, bool_t yesno){
+	f->synchronous_notifies=yesno;
+}
+
 void ms_filter_destroy(MSFilter *f){
 	if (f->desc->uninit!=NULL)
 		f->desc->uninit(f);
@@ -192,10 +202,55 @@ void ms_filter_destroy(MSFilter *f){
 	ms_free(f);
 }
 
+#ifdef DEBUG
+
+static long filter_get_cur_time(void *unused)
+{
+#if defined(_WIN32_WCE)
+	DWORD timemillis = GetTickCount();
+	return timemillis;
+#elif defined(WIN32)
+	return timeGetTime() ;
+#elif defined(__MACH__) && defined(__GNUC__) && (__GNUC__ >= 3)
+	struct timeval tv;
+	gettimeofday(&tv, NULL);
+	return (tv.tv_sec*1000LL) + (tv.tv_usec/1000LL);
+#elif defined(__MACH__)
+	struct timespec ts;
+	struct timeb time_val;
+
+	ftime (&time_val);
+	ts.tv_sec = time_val.time;
+	ts.tv_nsec = time_val.millitm * 1000000;
+	return (ts.tv_sec*1000LL) + (ts.tv_nsec/1000000LL);
+#else
+	struct timespec ts;
+	if (clock_gettime(CLOCK_MONOTONIC,&ts)<0){
+		fprintf(stderr, "clock_gettime() doesn't work: %s",strerror(errno));
+	}
+	return (ts.tv_sec*1000LL) + (ts.tv_nsec/1000000LL);
+#endif
+}
+#endif
 
 void ms_filter_process(MSFilter *f){
 	ms_debug("Executing process of filter %s:%p",f->desc->name,f);
+#ifdef DEBUG
+	long start,stop;
+	start = filter_get_cur_time(NULL);
+#endif
 	f->desc->process(f);
+#ifdef DEBUG
+	stop = filter_get_cur_time(NULL);
+	if(stop-start > 10)
+	{
+		ms_warning("%s take too much time:%ldms\n",f->desc->name,stop-start);
+	}
+	else
+	{
+		ms_debug("%s take:%ldms\n",f->desc->name,stop-start);
+	}
+#endif
 }
 
 void ms_filter_preprocess(MSFilter *f, struct _MSTicker *t){
@@ -220,15 +275,6 @@ bool_t ms_filter_inputs_have_data(MSFilter *f){
 	return FALSE;
 }
 
-void ms_filter_notify(MSFilter *f, unsigned int id, void *arg){
-	if (f->notify!=NULL)
-		f->notify(f->notify_ud,id,arg);
-}
-
-void ms_filter_notify_no_arg(MSFilter *f, unsigned int id){
-	if (f->notify!=NULL)
-		f->notify(f->notify_ud,id,NULL);
-}
 
 
 static void find_filters(MSList **filters, MSFilter *f ){
