@@ -180,7 +180,8 @@ static void enc_preprocess(MSFilter *f){
 	if (s->maxbitrate>0){
 		/* convert from network bitrate to codec bitrate:*/
 		/* ((nbr/(50*8)) -20-12-8)*50*8*/
-		int cbr=(int)( (((float)s->maxbitrate/(50.0*8))-20-12-8)*50*8);
+		int cbr=(int)( ((((float)s->maxbitrate)/(50.0*8))-20-12-8)*50*8);
+		ms_message("Setting maxbitrate=%i to speex encoder.",cbr);
 		if (speex_encoder_ctl(s->state,SPEEX_SET_BITRATE,&cbr)!=0){
 			ms_error("Could not set maxbitrate %i to speex encoder.",s->bitrate);
 		}
@@ -392,6 +393,7 @@ typedef struct DecState{
 	int rate;
 	int penh;
 	int frsz;
+	uint32_t last_ts;
 	void *state;
 } DecState;
 
@@ -401,6 +403,7 @@ static void dec_init(MSFilter *f){
 	s->frsz=0;
 	s->state=NULL;
 	s->penh=1;
+	s->last_ts=0;
 	f->data=s;
 }
 
@@ -456,35 +459,48 @@ static void dec_process(MSFilter *f){
 	DecState *s=(DecState*)f->data;
 	mblk_t *im;
 	mblk_t *om;
-	int err;
-	int frame_per_packet;
+	int err=-2;
 	SpeexBits bits;
 	int bytes=s->frsz*2;
+	
 	speex_bits_init(&bits);
 	while((im=ms_queue_get(f->inputs[0]))!=NULL){
+		uint32_t curts=mblk_get_timestamp_info(im);
+		int rem_bits=(im->b_wptr-im->b_rptr)*8;
+		
 		speex_bits_reset(&bits);
 		speex_bits_read_from(&bits,(char*)im->b_rptr,im->b_wptr-im->b_rptr);
-		om=allocb(bytes*7,0);
-		/* support for multiple frame (max=7 frames???) in one RTP packet */
-        for (frame_per_packet=0;frame_per_packet<7;frame_per_packet++)
-        {
-            int i;
-			err=speex_decode_int(s->state,&bits,(int16_t*)(om->b_wptr+(frame_per_packet*320)));
-            
-            i = speex_bits_remaining(&bits);
-			if (i<10) /* this seems to work: don't know why. */
-                break;
-        }
-		if (err==0){
-			om->b_wptr+=bytes*(frame_per_packet+1);
-			ms_queue_put(f->outputs[0],om);
-		}else {
-			if (err==-1)
-				ms_warning("speex end of stream");
-			else if (err==-2)
-				ms_warning("speex corrupted stream");
-			freemsg(om);
+		
+
+#if 0 /* does not work very well, revisit later*/
+		/* packet loss concealment stuff, for one packet lost 
+			A better algorithm would be needed to accomodate more
+		*/
+		if (s->last_ts!=0){
+			int ts_diff=(int)(curts-s->last_ts);
+			if ( ts_diff>160){
+				ms_message("Packet missing, trying loss concealment, ts_diff=%i",ts_diff);
+				err=speex_decode_int(s->state,NULL,(int16_t*)om->b_wptr);
+				om->b_wptr+=320;
+			}
 		}
+#endif
+		/* support for multiple frame  in one RTP packet */
+ 		do{
+			om=allocb(bytes,0);
+			err=speex_decode_int(s->state,&bits,(int16_t*)om->b_wptr);
+			om->b_wptr+=bytes;
+			s->last_ts=curts+s->frsz;
+			if (err==0){
+				ms_queue_put(f->outputs[0],om);
+			}else {
+				if (err==-1)
+					ms_warning("speex end of stream");
+				else if (err==-2)
+					ms_warning("speex corrupted stream");
+				freemsg(om);
+			}
+		}while((rem_bits= speex_bits_remaining(&bits))>10);
 		freemsg(im);
 	}
 	speex_bits_destroy(&bits);
