@@ -75,8 +75,18 @@ typedef pthread_cond_t ortp_cond_t;
 #pragma warning(disable : 1469)		// "cc" clobber ignored
 #endif
 
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+
 int __ortp_thread_join(ortp_thread_t thread, void **ptr);
 int __ortp_thread_create(pthread_t *thread, pthread_attr_t *attr, void * (*routine)(void*), void *arg);
+
+#ifdef __cplusplus
+}
+#endif
+
 #define ortp_thread_create	__ortp_thread_create
 #define ortp_thread_join	__ortp_thread_join
 #define ortp_thread_exit	pthread_exit
@@ -96,6 +106,9 @@ int __ortp_thread_create(pthread_t *thread, pthread_attr_t *attr, void * (*routi
 #define getSocketError() strerror(errno)
 #define getSocketErrorCode() (errno)
 
+#define ortp_log10f(x)	log10f(x)
+
+
 #else
 /*********************************/
 /* definitions for WIN32 flavour */
@@ -106,7 +119,7 @@ int __ortp_thread_create(pthread_t *thread, pthread_attr_t *attr, void * (*routi
 #include <stdarg.h>
 #include <winsock2.h>
 #include <ws2tcpip.h>
-
+#include <io.h>
 
 #ifdef _MSC_VER
 #pragma push_macro("_WINSOCKAPI_")
@@ -137,7 +150,7 @@ typedef HANDLE ortp_thread_t;
 
 #define ortp_thread_create	WIN_thread_create
 #define ortp_thread_join	WIN_thread_join
-#define ortp_thread_exit(arg)		
+#define ortp_thread_exit(arg)
 #define ortp_mutex_init		WIN_mutex_init
 #define ortp_mutex_lock		WIN_mutex_lock
 #define ortp_mutex_unlock	WIN_mutex_unlock
@@ -153,12 +166,12 @@ typedef HANDLE ortp_thread_t;
 extern "C"
 {
 #endif
-	
+
 int WIN_mutex_init(ortp_mutex_t *m, void *attr_unused);
 int WIN_mutex_lock(ortp_mutex_t *mutex);
 int WIN_mutex_unlock(ortp_mutex_t *mutex);
 int WIN_mutex_destroy(ortp_mutex_t *mutex);
-int WIN_thread_create(ortp_thread_t *t, void *attr_unused, void *(*func)(void*), void *arg); 
+int WIN_thread_create(ortp_thread_t *t, void *attr_unused, void *(*func)(void*), void *arg);
 int WIN_thread_join(ortp_thread_t thread, void **unused);
 int WIN_cond_init(ortp_cond_t *cond, void *attr_unused);
 int WIN_cond_wait(ortp_cond_t * cond, ortp_mutex_t * mutex);
@@ -173,6 +186,32 @@ int WIN_cond_destroy(ortp_cond_t * cond);
 #define SOCKET_OPTION_VALUE	char *
 #define inline			__inline
 
+#if defined(_WIN32_WCE)
+
+#define ortp_log10f(x)		(float)log10 ((double)x)
+
+#ifdef assert
+	#undef assert
+#endif /*assert*/
+#define assert(exp)	((void)0)
+
+#ifdef errno
+	#undef errno
+#endif /*errno*/
+#define  errno GetLastError()
+#ifdef strerror
+		#undef strerror
+#endif /*strerror*/
+const char * ortp_strerror(DWORD value);
+#define strerror ortp_strerror
+
+
+#else /*_WIN32_WCE*/
+
+#define ortp_log10f(x)	log10f(x)
+
+#endif
+
 const char *getWinSocketError(int error);
 #define getSocketErrorCode() WSAGetLastError()
 #define getSocketError() getWinSocketError(WSAGetLastError())
@@ -220,8 +259,8 @@ typedef struct _OrtpMemoryFunctions{
 
 void ortp_set_memory_functions(OrtpMemoryFunctions *functions);
 
-#define ortp_new(type,count)	ortp_malloc(sizeof(type)*(count))
-#define ortp_new0(type,count)	ortp_malloc0(sizeof(type)*(count))
+#define ortp_new(type,count)	(type*)ortp_malloc(sizeof(type)*(count))
+#define ortp_new0(type,count)	(type*)ortp_malloc0(sizeof(type)*(count))
 
 int close_socket(ortp_socket_t sock);
 int set_non_blocking_socket(ortp_socket_t sock);
@@ -230,7 +269,10 @@ char *ortp_strndup(const char *str,int n);
 char *ortp_strdup_printf(const char *fmt,...);
 char *ortp_strdup_vprintf(const char *fmt, va_list ap);
 
-/* portable named pipes */
+int ortp_file_exist(const char *pathname);
+
+/* portable named pipes  and shared memory*/
+#if !defined(_WIN32_WCE)
 #ifdef WIN32
 typedef HANDLE ortp_pipe_t;
 #define ORTP_PIPE_INVALID INVALID_HANDLE_VALUE
@@ -255,9 +297,14 @@ int ortp_client_pipe_close(ortp_pipe_t sock);
 int ortp_pipe_read(ortp_pipe_t p, uint8_t *buf, int len);
 int ortp_pipe_write(ortp_pipe_t p, const uint8_t *buf, int len);
 
+void *ortp_shm_open(unsigned int keyid, int size, int create);
+void ortp_shm_close(void *memory);
+
+#endif
 
 #ifdef __cplusplus
 }
+
 #endif
 
 
