@@ -67,7 +67,7 @@ typedef struct rtp_stats
 	uint64_t recv; 		/* bytes of payload received and delivered in time to the application */
 	uint64_t hw_recv;		/* bytes of payload received */
 	uint64_t packet_recv;	/* number of packets received */
-	uint64_t unavaillable;	/* packets not availlable when they were queried */
+	uint64_t unavaillable;	/* totally useless*/
 	uint64_t outoftime;		/* number of packets that were received too late */
 	uint64_t cum_packet_loss; /* cumulative number of packet lost */
 	uint64_t bad;			/* packets that did not appear to be RTP */
@@ -85,6 +85,10 @@ typedef struct rtp_stats
 #define TIME_IS_STRICTLY_NEWER_THAN(t1,t2) RTP_TIMESTAMP_IS_STRICTLY_NEWER_THAN(t1,t2)
 
 
+#ifdef __cplusplus
+extern "C"{
+#endif
+
 /* packet api */
 /* the first argument is a mblk_t. The header is supposed to be not splitted  */
 #define rtp_set_markbit(mp,value)		((rtp_header_t*)((mp)->b_rptr))->markbit=(value)
@@ -102,5 +106,10 @@ void rtp_add_csrc(mblk_t *mp ,uint32_t csrc);
 #define rtp_get_cc(mp)		(((rtp_header_t*)((mp)->b_rptr))->cc)
 #define rtp_get_csrc(mp, idx)		(((rtp_header_t*)((mp)->b_rptr))->csrc[idx])
 
+int rtp_get_payload(mblk_t *packet, unsigned char **start);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif
