@@ -56,16 +56,19 @@ typedef struct _JBParameters{
 	int nom_size; /**< idem */
 	int max_size; /**< idem */
 	bool_t adaptive;
+	bool_t pad[3];
+	int max_packets; /**< max number of packets allowed to be queued in the jitter buffer */
 } JBParameters;
 
 typedef struct _JitterControl
 {
+	int count;
 	int jitt_comp;   /* the user jitt_comp in miliseconds*/
 	int jitt_comp_ts; /* the jitt_comp converted in rtp time (same unit as timestamp) */
 	int adapt_jitt_comp_ts;
-	float slide;
+	int64_t slide;
+	int64_t prev_slide;
 	float jitter;
-	int count;
 	int olddiff;
 	float inter_jitter;	/* interarrival jitter as defined in the RFC */
 	int corrective_step;
@@ -96,7 +99,7 @@ typedef struct _RtpTransport
 typedef struct _RtpStream
 {
 	ortp_socket_t socket;
-        struct _RtpTransport *tr; 
+	struct _RtpTransport *tr; 
 	int sockfamily;
 	int max_rq_size;
 	int time_jump;
@@ -111,6 +114,8 @@ typedef struct _RtpStream
 	struct sockaddr_in rem_addr;
 #endif
 	int rem_addrlen;
+	void *QoSHandle;
+	unsigned long QoSFlowID;
 	JitterControl jittctl;
 	uint32_t snd_time_offset;/*the scheduler time when the application send its first timestamp*/	
 	uint32_t snd_ts_offset;	/* the first application timestamp sent by the application */
@@ -119,8 +124,6 @@ typedef struct _RtpStream
 	uint32_t rcv_time_offset; /*the scheduler time when the application ask for its first timestamp*/
 	uint32_t rcv_ts_offset;  /* the first stream timestamp */
 	uint32_t rcv_query_ts_offset;	/* the first user timestamp asked by the application */
-	uint32_t rcv_diff_ts;	/* difference between the first user timestamp and first stream timestamp */
-	uint32_t hwrcv_diff_ts;
 	uint32_t rcv_last_ts;	/* the last stream timestamp got by the application */
 	uint32_t rcv_last_app_ts; /* the last application timestamp asked by the application */	
 	uint32_t rcv_last_ret_ts; /* the timestamp of the last sample returned (only for continuous audio)*/
@@ -134,6 +137,7 @@ typedef struct _RtpStream
 	uint32_t last_rtcp_report_snt_s;	/* the time of the last rtcp report sent, in send timestamp unit */
 	uint32_t rtcp_report_snt_interval; /* the interval in timestamp unit between rtcp report sent */
 	uint32_t last_rtcp_packet_count; /*the sender's octet count in the last sent RTCP SR*/
+	uint32_t sent_payload_bytes; /*used for RTCP sender reports*/
 	unsigned int sent_bytes; /* used for bandwidth estimation */
 	struct timeval send_bw_start; /* used for bandwidth estimation */
 	unsigned int recv_bytes; /* used for bandwidth estimation */
@@ -141,14 +145,16 @@ typedef struct _RtpStream
 	rtp_stats_t stats;
 	int recv_errno;
 	int send_errno;
+	int snd_socket_size;
+	int rcv_socket_size;
+	int ssrc_changed_thres;
 }RtpStream;
 
 typedef struct _RtcpStream
 {
 	ortp_socket_t socket;
 	int sockfamily;
-        struct _RtpTransport *tr; 
-
+	struct _RtpTransport *tr; 
 	mblk_t *cached_mp;
 #ifdef ORTP_INET6
 	struct sockaddr_storage rem_addr;
@@ -194,6 +200,7 @@ struct _RtpSession
 	RtpSignalTable on_rtcp_bye;
 	struct _OList *signal_tables;
 	struct _OList *eventqs;
+	msgb_allocator_t allocator;
 	RtpStream rtp;
 	RtcpStream rtcp;
 	RtpSessionMode mode;
@@ -205,7 +212,7 @@ struct _RtpSession
 	void * user_data;
 	/* FIXME: Should be a table for all session participants. */
 	struct timeval last_recv_time; /* Time of receiving the RTP/RTCP packet. */
-
+	mblk_t *pending;
 	/* telephony events extension */
 	mblk_t *current_tev;		/* the pending telephony events */
 	mblk_t *sd;
@@ -253,6 +260,10 @@ bool_t rtp_session_adaptive_jitter_compensation_enabled(RtpSession *session);
 void rtp_session_set_time_jump_limit(RtpSession *session, int miliseconds);
 int rtp_session_set_local_addr(RtpSession *session,const char *addr, int port);
 int rtp_session_get_local_port(const RtpSession *session);
+
+int
+rtp_session_set_remote_addr_full (RtpSession * session, const char * addr, int rtp_port, int rtcp_port);
+/*same as previous function, old name:*/
 int rtp_session_set_remote_addr_and_port (RtpSession * session, const char * addr, int rtp_port, int rtcp_port);
 int rtp_session_set_remote_addr(RtpSession *session,const char *addr, int port);
 /* alternatively to the set_remote_addr() and set_local_addr(), an application can give
@@ -293,6 +304,8 @@ void rtp_session_set_connected_mode(RtpSession *session, bool_t yesno);
 
 void rtp_session_enable_rtcp(RtpSession *session, bool_t yesno);
 
+void rtp_session_set_ssrc_changed_threshold(RtpSession *session, int numpackets);
+
 /*low level recv and send functions */
 mblk_t * rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts);
 mblk_t * rtp_session_create_packet(RtpSession *session,int header_size, const uint8_t *payload, int payload_size);
@@ -329,6 +342,8 @@ void rtp_session_set_data(RtpSession *session, void *data);
 void *rtp_session_get_data(const RtpSession *session);
 
 void rtp_session_set_recv_buf_size(RtpSession *session, int bufsize);
+void rtp_session_set_rtp_socket_send_buffer_size(RtpSession * session, unsigned int size);
+void rtp_session_set_rtp_socket_recv_buffer_size(RtpSession * session, unsigned int size);
 
 /* in use with the scheduler to convert a timestamp in scheduler time unit (ms) */
 uint32_t rtp_session_ts_to_time(RtpSession *session,uint32_t timestamp);
