@@ -31,7 +31,7 @@
 #include "rtpsession_priv.h"
 #include <math.h>
 
-#define JC_BETA 0.01	
+#define JC_BETA 0.01
 #define JC_GAMMA (JC_BETA)
 
 #include "jitterctl.h"
@@ -39,6 +39,7 @@
 void jitter_control_init(JitterControl *ctl, int base_jiitt_time, PayloadType *payload){
 	ctl->count=0;
 	ctl->slide=0;
+	ctl->prev_slide=0;
 	ctl->jitter=0;
 	ctl->inter_jitter=0;
 	ctl->slide=0;
@@ -65,15 +66,22 @@ void jitter_control_set_payload(JitterControl *ctl, PayloadType *pt){
 
 void jitter_control_dump_stats(JitterControl *ctl){
 	ortp_message("JitterControl:\n\tslide=%g,jitter=%g,count=%i",
-			ctl->slide,ctl->jitter, ctl->count);
+			(double)ctl->slide,ctl->jitter, ctl->count);
 }
 
-
+/*the goal of this method is to compute "corrective_slide": a timestamp unit'd value to be added
+ to recv timestamp to make them reflect the instant they are delivered by the jitter buffer. */
 void jitter_control_update_corrective_slide(JitterControl *ctl){
 	int tmp;
-	tmp=(int)(ctl->slide)-ctl->corrective_slide;
-	if (tmp>ctl->corrective_step) ctl->corrective_slide+=ctl->corrective_step;
-	else if (tmp<-ctl->corrective_step) ctl->corrective_slide-=ctl->corrective_step;
+	tmp=(int)(ctl->slide)-ctl->prev_slide;
+	if (tmp>ctl->corrective_step) {
+		ctl->corrective_slide+=ctl->corrective_step;
+		ctl->prev_slide=ctl->slide+ctl->corrective_step;
+	}
+	else if (tmp<-ctl->corrective_step) {
+		ctl->corrective_slide-=ctl->corrective_step;
+		ctl->prev_slide=ctl->slide-ctl->corrective_step;
+	}
 }
 
 /*
@@ -84,42 +92,43 @@ void jitter_control_update_corrective_slide(JitterControl *ctl){
 	jitter is added to the initial jitt_comp_time value. It compensates bursty packets arrival (packets
 	not arriving at regular interval ).
 */
-void jitter_control_new_packet(JitterControl *ctl, uint32_t packet_ts, uint32_t cur_str_ts, int32_t * slide, int32_t *safe_delay){
-	int diff=packet_ts - cur_str_ts;
-	float gap;
+void jitter_control_new_packet(JitterControl *ctl, uint32_t packet_ts, uint32_t cur_str_ts){
+	int64_t diff=(int64_t)packet_ts - (int64_t)cur_str_ts;
+	double gap,slide;
 	int d;
 	//printf("diff=%g\n",diff);
 	if (ctl->count==0){
-		ctl->slide=diff;
+		slide=ctl->slide=ctl->prev_slide=diff;
+		ctl->olddiff=diff;
 		ctl->jitter=0;
 	}else{
-		ctl->slide= (float) ((ctl->slide*(1-JC_BETA)) + ((float)diff*JC_BETA));
+		slide=((double)ctl->slide*(1-JC_BETA)) + ((double)diff*JC_BETA);
 	}
-	gap=(float) fabs((float)diff - ctl->slide);
+	gap=(double)diff - slide;
+	gap=gap<0 ? -gap : 0; /*compute only for late packets*/
 	ctl->jitter=(float) ((ctl->jitter*(1-JC_GAMMA)) + (gap*JC_GAMMA));
 	d=diff-ctl->olddiff;
 	ctl->inter_jitter=(float) (ctl->inter_jitter+ (( (float)abs(d) - ctl->inter_jitter)*(1/16.0)));
 	ctl->olddiff=diff;
 	ctl->count++;
 	if (ctl->adaptive){
+		
 		if (ctl->count%50==0) {
+			ctl->adapt_jitt_comp_ts=(int) MAX(ctl->jitt_comp_ts,2*ctl->jitter);
 			/*jitter_control_dump_stats(ctl);*/
 		}
-		/* the following is nearly equivalent, but maybe it consumes more CPU: ?*/
-		/*ctl->corrective_slide=(((int)ctl->slide)/ctl->corrective_step)*ctl->corrective_step;*/
 		
-		ctl->adapt_jitt_comp_ts=(int) MAX(ctl->jitt_comp_ts,ctl->jitter);
-		
-		*slide=(int32_t)ctl->slide;
-		*safe_delay=(int32_t)ctl->adapt_jitt_comp_ts;
+		ctl->slide=slide;
 	}else {
-		*slide=0;
-		*safe_delay=(int32_t)ctl->jitt_comp_ts;
+		/*ctl->slide and jitter size are not updated*/
 	}
 	return ;
 }
 
 
+
+
+
 /**
  *rtp_session_set_jitter_compensation:
  *@session: a RtpSession
@@ -159,6 +168,7 @@ void rtp_session_set_jitter_buffer_params(RtpSession *session, const JBParameter
 	/* FIXME min_size and max_size to be implemented */
 	rtp_session_set_jitter_compensation(session,par->nom_size);
 	jitter_control_enable_adaptive(&session->rtp.jittctl,par->adaptive);
+	session->rtp.max_rq_size=par->max_packets;
 }
 
 void rtp_session_get_jitter_buffer_params(RtpSession *session, JBParameters *par){
@@ -167,5 +177,6 @@ void rtp_session_get_jitter_buffer_params(RtpSession *session, JBParameters *par
 	par->nom_size=nom_size;
 	par->max_size=-1;
 	par->adaptive=session->rtp.jittctl.adaptive;
+	par->max_packets=session->rtp.max_rq_size;
 }
 
