@@ -21,7 +21,7 @@
 
 #if defined(WIN32) || defined(_WIN32_WCE)
 #include "ortp-config-win32.h"
-#else
+#elif HAVE_CONFIG_H
 #include "ortp-config.h"
 #endif
 #include "ortp/port.h"
@@ -32,6 +32,10 @@
 #include <process.h>
 #endif
 
+#ifdef HAVE_SYS_SHM_H
+#include <sys/shm.h>
+#endif
+
 static void *ortp_libc_malloc(size_t sz){
 	return malloc(sz);
 }
@@ -100,7 +104,7 @@ char * ortp_strdup(const char *tmp){
  */
 int set_non_blocking_socket (ortp_socket_t sock)
 {
-	
+
 
 #if	!defined(_WIN32) && !defined(_WIN32_WCE)
 	return fcntl (sock, F_SETFL, O_NONBLOCK);
@@ -124,7 +128,23 @@ int close_socket(ortp_socket_t sock){
 #endif
 }
 
-
+#if defined (_WIN32_WCE) || defined(_MSC_VER)
+int ortp_file_exist(const char *pathname) {
+	FILE* fd;
+	if (pathname==NULL) return -1;
+	fd=fopen(pathname,"r");
+	if (fd==NULL) {
+		return -1;
+	} else {
+		fclose(fd);
+		return 0;
+	}
+}
+#else
+int ortp_file_exist(const char *pathname) {
+	return access(pathname,F_OK);
+}
+#endif /*_WIN32_WCE*/
 
 #if	!defined(_WIN32) && !defined(_WIN32_WCE)
 	/* Use UNIX inet_aton method */
@@ -132,10 +152,10 @@ int close_socket(ortp_socket_t sock){
 	int inet_aton (const char * cp, struct in_addr * addr)
 	{
 		unsigned long retval;
-		
+
 		retval = inet_addr (cp);
 
-		if (retval == INADDR_NONE) 
+		if (retval == INADDR_NONE)
 		{
 			return -1;
 		}
@@ -180,7 +200,7 @@ int __ortp_thread_create(pthread_t *thread, pthread_attr_t *attr, void * (*routi
 #if	defined(_WIN32) || defined(_WIN32_WCE)
 
 int WIN_mutex_init(ortp_mutex_t *mutex, void *attr)
-{	
+{
 	*mutex=CreateMutex(NULL, FALSE, NULL);
 	return 0;
 }
@@ -248,7 +268,7 @@ int WIN_cond_init(ortp_cond_t *cond, void *attr)
 int WIN_cond_wait(ortp_cond_t* hCond, ortp_mutex_t * hMutex)
 {
 	//gulp: this is not very atomic ! bug here ?
-	WIN_mutex_unlock(hMutex);	
+	WIN_mutex_unlock(hMutex);
 	WaitForSingleObject(*hCond, INFINITE);
 	WIN_mutex_lock(hMutex);
 	return 0;
@@ -276,6 +296,21 @@ int WIN_cond_destroy(ortp_cond_t * hCond)
 #if defined(_WIN32_WCE)
 #include <time.h>
 
+const char * ortp_strerror(DWORD value) {
+	static TCHAR msgBuf[256];
+	FormatMessage(
+			FORMAT_MESSAGE_FROM_SYSTEM |
+			FORMAT_MESSAGE_IGNORE_INSERTS,
+			NULL,
+			value,
+			0, // Default language
+			(LPTSTR) &msgBuf,
+			0,
+			NULL
+	);
+	return (const char *)msgBuf;
+}
+
 int
 gettimeofday (struct timeval *tv, void *tz)
 {
@@ -287,19 +322,19 @@ gettimeofday (struct timeval *tv, void *tz)
 
 #else
 
-int gettimeofday (struct timeval *tv, void* tz) 
-{ 
-	union 
-	{ 
-		__int64 ns100; /*time since 1 Jan 1601 in 100ns units */ 
-		FILETIME fileTime; 
-	} now; 
+int gettimeofday (struct timeval *tv, void* tz)
+{
+	union
+	{
+		__int64 ns100; /*time since 1 Jan 1601 in 100ns units */
+		FILETIME fileTime;
+	} now;
 
-	GetSystemTimeAsFileTime (&now.fileTime); 
-	tv->tv_usec = (long) ((now.ns100 / 10LL) % 1000000LL); 
-	tv->tv_sec = (long) ((now.ns100 - 116444736000000000LL) / 10000000LL); 
-	return (0); 
-} 
+	GetSystemTimeAsFileTime (&now.fileTime);
+	tv->tv_usec = (long) ((now.ns100 / 10LL) % 1000000LL);
+	tv->tv_sec = (long) ((now.ns100 - 116444736000000000LL) / 10000000LL);
+	return (0);
+}
 
 #endif
 
@@ -405,7 +440,40 @@ int ortp_client_pipe_close(ortp_socket_t sock){
 	return close(sock);
 }
 
+#ifdef HAVE_SYS_SHM_H
+
+void *ortp_shm_open(unsigned int keyid, int size, int create){
+	key_t key=keyid;
+	void *mem;
+	int fd=shmget(key,size,create ? (IPC_CREAT | 0666) : 0666);
+	if (fd==-1){
+		printf("shmget failed: %s\n",strerror(errno));
+		return NULL;
+	}
+	mem=shmat(fd,NULL,0);
+	if (mem==(void*)-1){
+		printf("shmat() failed: %s", strerror(errno));
+		return NULL;
+	}
+	return mem;
+}
+
+void ortp_shm_close(void *mem){
+	shmdt(mem);
+}
+
 #else
+void *ortp_shm_open(unsigned int keyid, int size, int create){
+	ortp_error("No shared memory support for this OS.");
+}
+
+void ortp_shm_close(void *mem){
+}
+
+
+#endif
+
+#elif defined(WIN32) && !defined(_WIN32_WCE)
 
 static char *make_pipe_name(const char *name){
 	return ortp_strdup_printf("\\\\.\\pipe\\%s",name);
@@ -462,15 +530,15 @@ int ortp_server_pipe_close(ortp_pipe_t spipe){
 
 ortp_pipe_t ortp_client_pipe_connect(const char *name){
 	char *pipename=make_pipe_name(name);
-	ortp_pipe_t hpipe = CreateFile( 
-         pipename,   // pipe name 
-         GENERIC_READ |  // read and write access 
-         GENERIC_WRITE, 
-         0,              // no sharing 
+	ortp_pipe_t hpipe = CreateFile(
+         pipename,   // pipe name
+         GENERIC_READ |  // read and write access
+         GENERIC_WRITE,
+         0,              // no sharing
          NULL,           // default security attributes
-         OPEN_EXISTING,  // opens existing pipe 
-         0,              // default attributes 
-         NULL);          // no template file 
+         OPEN_EXISTING,  // opens existing pipe
+         0,              // default attributes
+         NULL);          // no template file
 	ortp_free(pipename);
 	return hpipe;
 }
@@ -496,4 +564,11 @@ int ortp_client_pipe_close(ortp_pipe_t sock){
 	return CloseHandle(sock);
 }
 
+void *ortp_shm_open(unsigned int keyid, int size, int create{
+	return NULL;
+}
+
+void ortp_shm_close(void *mem){
+}
+
 #endif
