@@ -20,7 +20,7 @@
 
 #if defined(WIN32) || defined(_WIN32_WCE)
 #include "ortp-config-win32.h"
-#else
+#elif HAVE_CONFIG_H
 #include "ortp-config.h"
 #endif
 
@@ -32,6 +32,12 @@
 #include "utils.h"
 #include "rtpsession_priv.h"
 
+#if (_WIN32_WINNT >= 0x0600)
+#include <delayimp.h>
+#undef ExternC /* avoid redefinition... */
+#include <QOS2.h>
+#endif
+
 extern mblk_t *rtcp_create_simple_bye_packet(uint32_t ssrc, const char *reason);
 extern int rtcp_sr_init(RtpSession *session, char *buf, int size);
 extern int rtcp_rr_init(RtpSession *session, char *buf, int size);
@@ -214,6 +220,11 @@ void
 rtp_session_init (RtpSession * session, int mode)
 {
 	JBParameters jbp;
+	if (session == NULL) 
+	{
+	    ortp_debug("rtp_session_init: Invalid paramter (session=NULL)");
+	    return;
+	}
 	memset (session, 0, sizeof (RtpSession));
 	session->mode = (RtpSessionMode) mode;
 	if ((mode == RTP_SESSION_RECVONLY) || (mode == RTP_SESSION_SENDRECV))
@@ -241,6 +252,7 @@ rtp_session_init (RtpSession * session, int mode)
 #else
 	session->rtp.snd_socket_size=session->rtp.rcv_socket_size=65536;
 #endif
+	session->rtp.ssrc_changed_thres=50;
 	session->dscp=RTP_DEFAULT_DSCP;
 	session->multicast_ttl=RTP_DEFAULT_MULTICAST_TTL;
 	session->multicast_loopback=RTP_DEFAULT_MULTICAST_LOOPBACK;
@@ -293,7 +305,12 @@ rtp_session_new (int mode)
 {
 	RtpSession *session;
 	session = (RtpSession *) ortp_malloc (sizeof (RtpSession));
-	rtp_session_init (session, mode);
+	if (session == NULL)
+	{
+	    ortp_error("rtp_session_new: Memory allocation failed");
+	    return NULL;
+	}
+      rtp_session_init (session, mode);
 	return session;
 }
 
@@ -584,8 +601,8 @@ rtp_session_set_ssrc (RtpSession * session, uint32_t ssrc)
 void rtp_session_update_payload_type(RtpSession *session, int paytype){
 	/* check if we support this payload type */
 	PayloadType *pt=rtp_profile_get_payload(session->rcv.profile,paytype);
-	session->hw_recv_pt=paytype;
 	if (pt!=0){
+		session->hw_recv_pt=paytype;
 		ortp_message ("payload type changed to %i(%s) !",
 				 paytype,pt->mime_type);
 		payload_type_changed(session,pt);
@@ -910,8 +927,11 @@ extern void rtcp_parse(RtpSession *session, mblk_t *mp);
 
 
 static void payload_type_changed_notify(RtpSession *session, int paytype){
-	session->rcv.pt = paytype;
-	rtp_signal_table_emit (&session->on_payload_type_changed);	
+	PayloadType *pt = rtp_profile_get_payload(session->rcv.profile,paytype);
+	if (pt) {
+		session->rcv.pt = paytype;
+		rtp_signal_table_emit (&session->on_payload_type_changed);
+	}
 }
 
 
@@ -1255,10 +1275,7 @@ void rtp_session_release_sockets(RtpSession *session){
 	if (session->rtcp.socket>=0) close_socket (session->rtcp.socket);
 	session->rtp.socket=-1;
 	session->rtcp.socket=-1;
-	if (session->rtp.tr!=NULL)
-	  ortp_free(session->rtp.tr);
-	if (session->rtcp.tr!=NULL)
-	  ortp_free(session->rtcp.tr);
+	
 	session->rtp.tr = 0;
 	session->rtcp.tr = 0;
 
@@ -1322,6 +1339,53 @@ void rtp_session_uninit (RtpSession * session)
 
 	session->signal_tables = o_list_free(session->signal_tables);
 	msgb_allocator_uninit(&session->allocator);
+
+#if (_WIN32_WINNT >= 0x0600)
+	if (session->rtp.QoSFlowID != 0)
+    {
+		OSVERSIONINFOEX ovi;
+		memset(&ovi, 0, sizeof(ovi));
+		ovi.dwOSVersionInfoSize = sizeof(ovi);
+		GetVersionEx((LPOSVERSIONINFO) & ovi);
+
+		ortp_message("check OS support for qwave.lib: %i %i %i\n",
+					ovi.dwMajorVersion, ovi.dwMinorVersion, ovi.dwBuildNumber);
+		if (ovi.dwMajorVersion > 5) {
+
+			if (FAILED(__HrLoadAllImportsForDll("qwave.dll"))) {
+				ortp_warning("Failed to load qwave.dll: no QoS available\n" );
+			}
+			else
+			{
+				BOOL QoSResult;
+				QoSResult = QOSRemoveSocketFromFlow(session->rtp.QoSHandle, 
+													0, 
+													session->rtp.QoSFlowID, 
+													0);
+
+				if (QoSResult != TRUE){
+					ortp_error("QOSRemoveSocketFromFlow failed to end a flow with error %d\n", 
+							GetLastError());
+				}
+				session->rtp.QoSFlowID=0;
+			}
+		}
+    }
+
+    if (session->rtp.QoSHandle != NULL)
+    {
+        QOSCloseHandle(session->rtp.QoSHandle);
+		session->rtp.QoSHandle=NULL;
+    }
+#endif
+}
+
+/**
+ * Sets the number of packets containg a new SSRC that will trigger the
+ * "ssrc_changed" callback.
+**/
+void rtp_session_set_ssrc_changed_threshold(RtpSession *session, int numpackets){
+	session->rtp.ssrc_changed_thres=numpackets;
 }
 
 /**
