@@ -22,13 +22,18 @@
 #include "ortp/rtpsession.h"
 #include "rtpsession_priv.h"
 
-
 #if defined(WIN32) || defined(_WIN32_WCE)
 #include "ortp-config-win32.h"
-#else
+#elif HAVE_CONFIG_H
 #include "ortp-config.h" /*needed for HAVE_SYS_UIO_H */
 #endif
 
+#if (_WIN32_WINNT >= 0x0600)
+#include <delayimp.h>
+#undef ExternC
+#include <QOS2.h>
+#endif
+
 #ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
 #define USE_SENDMSG 1
@@ -318,7 +323,7 @@ int rtp_session_set_multicast_ttl(RtpSession *session, int ttl)
 					 (SOCKET_OPTION_VALUE)	   &session->multicast_ttl, sizeof(session->multicast_ttl));
 
  		} break;
-
+#ifdef ORTP_INET6
         case AF_INET6: {
 
 			retval= setsockopt(session->rtp.socket, IPPROTO_IPV6, IPV6_MULTICAST_HOPS, 
@@ -330,7 +335,7 @@ int rtp_session_set_multicast_ttl(RtpSession *session, int ttl)
 					 (SOCKET_OPTION_VALUE) &session->multicast_ttl, sizeof(session->multicast_ttl));
 
         } break;
-
+#endif
         default:
             retval=-1;
     }
@@ -394,7 +399,7 @@ int rtp_session_set_multicast_loopback(RtpSession *session, int yesno)
 						 (SOCKET_OPTION_VALUE)   &session->multicast_loopback, sizeof(session->multicast_loopback));
 
  		} break;
-
+#ifdef ORTP_INET6
         case AF_INET6: {
 
 			retval= setsockopt(session->rtp.socket, IPPROTO_IPV6, IPV6_MULTICAST_LOOP, 
@@ -406,7 +411,7 @@ int rtp_session_set_multicast_loopback(RtpSession *session, int yesno)
 				 (SOCKET_OPTION_VALUE)	&session->multicast_loopback, sizeof(session->multicast_loopback));
 
         } break;
-
+#endif
         default:
             retval=-1;
     }
@@ -444,6 +449,9 @@ int rtp_session_get_multicast_loopback(RtpSession *session)
 int rtp_session_set_dscp(RtpSession *session, int dscp){
 	int retval=0;
 	int tos;
+#if (_WIN32_WINNT >= 0x0600)
+	OSVERSIONINFOEX ovi;
+#endif
 
 	// Store new DSCP value if one is specified
 	if (dscp>=0) session->dscp = dscp;
@@ -451,27 +459,90 @@ int rtp_session_set_dscp(RtpSession *session, int dscp){
 	// Don't do anything if socket hasn't been created yet
 	if (session->rtp.socket < 0) return 0;
 
-	// DSCP value is in the upper six bits of the TOS field
-	tos = (session->dscp << 2) & 0xFC;
-	switch (session->rtp.sockfamily) {
-		case AF_INET:
-		retval = setsockopt(session->rtp.socket, IPPROTO_IP, IP_TOS, (SOCKET_OPTION_VALUE)&tos, sizeof(tos));
-		break;
+#if (_WIN32_WINNT >= 0x0600)
+	memset(&ovi, 0, sizeof(ovi));
+	ovi.dwOSVersionInfoSize = sizeof(ovi);
+	GetVersionEx((LPOSVERSIONINFO) & ovi);
+
+	ortp_message("check OS support for qwave.lib: %i %i %i\n",
+				ovi.dwMajorVersion, ovi.dwMinorVersion, ovi.dwBuildNumber);
+	if (ovi.dwMajorVersion > 5) {
+
+		if (FAILED(__HrLoadAllImportsForDll("qwave.dll"))) {
+			ortp_warning("Failed to load qwave.dll: no QoS available\n" );
+		}
+		else
+		{
+			if (session->dscp==0)
+				tos=QOSTrafficTypeBestEffort;
+			else if (session->dscp==0x8)
+				tos=QOSTrafficTypeBackground;
+			else if (session->dscp==0x28)
+				tos=QOSTrafficTypeAudioVideo;
+			else if (session->dscp==0x38)
+				tos=QOSTrafficTypeVoice;
+			else
+				tos=QOSTrafficTypeExcellentEffort; /* 0x28 */
+
+			if (session->rtp.QoSHandle==NULL) {
+				QOS_VERSION version;
+				BOOL QoSResult;
+
+				version.MajorVersion = 1;
+				version.MinorVersion = 0;
+
+				QoSResult = QOSCreateHandle(&version, &session->rtp.QoSHandle);
+
+				if (QoSResult != TRUE){
+					ortp_error("QOSCreateHandle failed to create handle with error %d\n",
+						GetLastError());
+					retval=-1;
+				}
+			}
+			if (session->rtp.QoSHandle!=NULL) {
+				BOOL QoSResult;
+				QoSResult = QOSAddSocketToFlow(
+					session->rtp.QoSHandle, 
+					session->rtp.socket,
+					(struct sockaddr*)&session->rtp.rem_addr,
+					tos, 
+					QOS_NON_ADAPTIVE_FLOW, 
+					&session->rtp.QoSFlowID);
+
+				if (QoSResult != TRUE){
+					ortp_error("QOSAddSocketToFlow failed to add a flow with error %d\n", 
+						GetLastError());
+					retval=-1;
+				}
+			}
+		}
+	} else {
+#endif
+		// DSCP value is in the upper six bits of the TOS field
+		tos = (session->dscp << 2) & 0xFC;
+		switch (session->rtp.sockfamily) {
+			case AF_INET:
+			retval = setsockopt(session->rtp.socket, IPPROTO_IP, IP_TOS, (SOCKET_OPTION_VALUE)&tos, sizeof(tos));
+			break;
 #ifdef ORTP_INET6
-	case AF_INET6:
+		case AF_INET6:
 #	ifdef IPV6_TCLASS /*seems not defined by my libc*/
-		retval = setsockopt(session->rtp.socket, IPPROTO_IPV6, IPV6_TCLASS,
-		 (SOCKET_OPTION_VALUE)&tos, sizeof(tos));
+			retval = setsockopt(session->rtp.socket, IPPROTO_IPV6, IPV6_TCLASS,
+			 (SOCKET_OPTION_VALUE)&tos, sizeof(tos));
 #	else
-		/*in case that works:*/
-		retval = setsockopt(session->rtp.socket, IPPROTO_IPV6, IP_TOS,
-		 (SOCKET_OPTION_VALUE)&tos, sizeof(tos));
+			/*in case that works:*/
+			retval = setsockopt(session->rtp.socket, IPPROTO_IPV6, IP_TOS,
+			 (SOCKET_OPTION_VALUE)&tos, sizeof(tos));
+#	endif
+			break;
 #endif
-		break;
-#endif
-	default:
-		retval=-1;
+		default:
+			retval=-1;
+		}
+#if (_WIN32_WINNT >= 0x0600)
 	}
+#endif
+
 	if (retval<0)
 		ortp_warning("Failed to set DSCP value on socket.");
 
@@ -730,10 +801,10 @@ void rtp_session_flush_sockets(RtpSession *session){
 	  }
 
 	if (session->rtp.socket>=0){
-		while (recvfrom(session->rtp.socket,trash,sizeof(trash),0,(struct sockaddr *)&from,&fromlen)>0){};
+		while (recvfrom(session->rtp.socket,(char*)trash,sizeof(trash),0,(struct sockaddr *)&from,&fromlen)>0){};
 	}
 	if (session->rtcp.socket>=0){
-		while (recvfrom(session->rtcp.socket,trash,sizeof(trash),0,(struct sockaddr*)&from,&fromlen)>0){};
+		while (recvfrom(session->rtcp.socket,(char*)trash,sizeof(trash),0,(struct sockaddr*)&from,&fromlen)>0){};
 	}
 }
 
@@ -811,7 +882,7 @@ rtp_session_rtp_send (RtpSession * session, mblk_t * m)
 #else
 		if (m->b_cont!=NULL)
 			msgpullup(m,-1);
-		error = sendto (sockfd, m->b_rptr, (int) (m->b_wptr - m->b_rptr),
+		error = sendto (sockfd, (char*)m->b_rptr, (int) (m->b_wptr - m->b_rptr),
 			 0,destaddr,destlen);
 #endif
 	}
@@ -855,7 +926,7 @@ rtp_session_rtcp_send (RtpSession * session, mblk_t * m)
 			if (m->b_cont!=NULL){
 				msgpullup(m,-1);
 			}
-			error = sendto (sockfd, m->b_rptr,
+			error = sendto (sockfd, (char*)m->b_rptr,
 			(int) (m->b_wptr - m->b_rptr), 0,
 			destaddr, destlen);
 #endif
@@ -896,12 +967,12 @@ rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 		mp=session->rtp.cached_mp;
 		bufsz=(int) (mp->b_datap->db_lim - mp->b_datap->db_base);
 		if (sock_connected){
-			error=recv(sockfd,mp->b_wptr,bufsz,0);
+			error=recv(sockfd,(char*)mp->b_wptr,bufsz,0);
 		}else if (rtp_session_using_transport(session, rtp)) 
 			error = (session->rtp.tr->t_recvfrom)(session->rtp.tr, mp, 0,
 				  (struct sockaddr *) &remaddr,
 				  &addrlen);
-		else error = recvfrom(sockfd, mp->b_wptr,
+		else error = recvfrom(sockfd, (char*)mp->b_wptr,
 				  bufsz, 0,
 				  (struct sockaddr *) &remaddr,
 				  &addrlen);
@@ -925,11 +996,11 @@ rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 		else
 		{
 		 	int errnum=getSocketErrorCode();
-
-			if (error == 0)
-			{
-				ortp_warning
-					("rtp_recv: strange... recv() returned zero.");
+			if (error == 0){
+				/*0 can be returned by RtpTransport functions in case of EWOULDBLOCK*/
+				/*we ignore it*/
+				/*ortp_warning
+					("rtp_recv: strange... recv() returned zero.");*/
 			}
 			else if (!is_would_block_error(errnum))
 			{
@@ -977,7 +1048,7 @@ rtp_session_rtcp_recv (RtpSession * session)
 		
 		mp=session->rtcp.cached_mp;
 		if (sock_connected){
-			error=recv(session->rtcp.socket,mp->b_wptr,RTCP_MAX_RECV_BUFSIZE,0);
+			error=recv(session->rtcp.socket,(char*)mp->b_wptr,RTCP_MAX_RECV_BUFSIZE,0);
 		}else {
 			addrlen=sizeof (remaddr);
 
@@ -986,7 +1057,7 @@ rtp_session_rtcp_recv (RtpSession * session)
 				  (struct sockaddr *) &remaddr,
 				  &addrlen);
 			else
-			  error=recvfrom (session->rtcp.socket, mp->b_wptr,
+			  error=recvfrom (session->rtcp.socket,(char*) mp->b_wptr,
 				  RTCP_MAX_RECV_BUFSIZE, 0,
 				  (struct sockaddr *) &remaddr,
 				  &addrlen);
