@@ -17,9 +17,9 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#if defined(WIN32) || defined(_WIN32_WCE)
+#if defined(_MSC_VER)  && (defined(WIN32) || defined(_WIN32_WCE))
 #include "ortp-config-win32.h"
-#else
+#elif HAVE_CONFIG_H
 #include "ortp-config.h"
 #endif
 #include "ortp/ortp.h"
