@@ -17,7 +17,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#include "ortp/port.h"
+#include "ortp/ortp.h"
 #include "ortp/rtp.h"
 #include "ortp/str_utils.h"
 #include "utils.h"
@@ -106,7 +106,7 @@ void freeb(mblk_t *mp)
 	
 	datab_unref(mp->b_datap);
 	ortp_free(mp);
-}	
+}
 
 void freemsg(mblk_t *mp)
 {
@@ -129,6 +129,8 @@ mblk_t *dupb(mblk_t *mp)
 	datab_ref(mp->b_datap);
 	newm=(mblk_t *) ortp_malloc(sizeof(mblk_t));
 	mblk_init(newm);
+	newm->reserved1=mp->reserved1;
+	newm->reserved2=mp->reserved2;
 	newm->b_datap=mp->b_datap;
 	newm->b_rptr=mp->b_rptr;
 	newm->b_wptr=mp->b_wptr;
@@ -309,3 +311,29 @@ mblk_t *concatb(mblk_t *mp, mblk_t *newm){
 	while(newm->b_cont!=NULL) newm=newm->b_cont;
 	return newm;
 }
+
+void msgb_allocator_init(msgb_allocator_t *a){
+	qinit(&a->q);
+}
+
+mblk_t *msgb_allocator_alloc(msgb_allocator_t *a, int size){
+	queue_t *q=&a->q;
+	mblk_t *m,*found=NULL;
+
+	/*lookup for an unused msgb (data block with ref count ==1)*/
+	for(m=qbegin(q);!qend(q,m);m=qnext(q,m)){
+		if (m->b_datap->db_ref==1 && m->b_datap->db_lim-m->b_datap->db_base>=size){
+			found=m;
+			break;
+		}
+	}
+	if (found==NULL){
+		found=allocb(size,0);
+		putq(q,found);
+	}
+	return dupb(found);
+}
+
+void msgb_allocator_uninit(msgb_allocator_t *a){
+	flushq(&a->q,-1);
+}
