@@ -1,5 +1,5 @@
  /*
-  The lprtplib LinPhone RTP library intends to provide basics for a RTP stack.
+  The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) stack..
   Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
 
   This library is free software; you can redistribute it and/or
@@ -39,7 +39,7 @@ void ssrc_cb(RtpSession *session)
 	printf("hey, the ssrc has changed !\n");
 }
 
-static char *help="usage: rtprecv  filename loc_port [--format format] [--soundcard]\n";
+static char *help="usage: rtprecv  filename loc_port [--format format] [--soundcard] [--noadapt] [--with-jitter <milliseconds>]\n";
 
 #define MULAW 0
 #define ALAW 1
@@ -85,16 +85,17 @@ int main(int argc, char*argv[])
 	int format=0;
 	int soundcard=0;
 	int sound_fd=0;
+	int jittcomp=40;
 	bool_t adapt=TRUE;
 	
 	/* init the lib */
 	if (argc<3){
-		printf(help);
+		printf("%s",help);
 		return -1;
 	}
 	local_port=atoi(argv[2]);
 	if (local_port<=0) {
-		printf(help);
+		printf("%s",help);
 		return -1;
 	}
 	for (i=3;i<argc;i++)
@@ -117,6 +118,13 @@ int main(int argc, char*argv[])
 		else if (strcmp(argv[i],"--soundcard")==0){
 			soundcard=1;
 		}
+		else if (strcmp(argv[i],"--with-jitter")==0){
+			i++;
+			if (i<argc){
+				jittcomp=atoi(argv[i]);
+				printf("Using a jitter buffer of %i milliseconds.\n",jittcomp);
+			}
+		}
 	}
 	
 	outfile=fopen(argv[1],"wb");
@@ -132,14 +140,16 @@ int main(int argc, char*argv[])
 	
 	ortp_init();
 	ortp_scheduler_init();
-	
+	ortp_set_log_level_mask(ORTP_DEBUG|ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR);
 	signal(SIGINT,stop_handler);
 	session=rtp_session_new(RTP_SESSION_RECVONLY);	
 	rtp_session_set_scheduling_mode(session,1);
 	rtp_session_set_blocking_mode(session,1);
 	rtp_session_set_local_addr(session,"0.0.0.0",atoi(argv[2]));
+	rtp_session_set_connected_mode(session,TRUE);
+	rtp_session_set_symmetric_rtp(session,TRUE);
 	rtp_session_enable_adaptive_jitter_compensation(session,adapt);
-	rtp_session_set_jitter_compensation(session,40);
+	rtp_session_set_jitter_compensation(session,jittcomp);
 	rtp_session_set_payload_type(session,0);
 	rtp_session_signal_connect(session,"ssrc_changed",(RtpCallback)ssrc_cb,0);
 	rtp_session_signal_connect(session,"ssrc_changed",(RtpCallback)rtp_session_reset,0);
